@@ -1,7 +1,9 @@
 package executor
 
 import (
+	"context"
 	"fmt"
+	"io"
 	"io/ioutil"
 	"log"
 	"net"
@@ -15,19 +17,19 @@ import (
 	"syscall"
 	"time"
 
+	"github.com/armon/circbuf"
 	"github.com/hashicorp/go-multierror"
 	"github.com/mitchellh/go-ps"
-	cgroupConfig "github.com/opencontainers/runc/libcontainer/configs"
 	"github.com/shirou/gopsutil/process"
 
 	"github.com/hashicorp/nomad/client/allocdir"
 	"github.com/hashicorp/nomad/client/driver/env"
 	"github.com/hashicorp/nomad/client/driver/logging"
 	"github.com/hashicorp/nomad/client/stats"
-	"github.com/hashicorp/nomad/command/agent/consul"
 	shelpers "github.com/hashicorp/nomad/helper/stats"
 	"github.com/hashicorp/nomad/nomad/structs"
-	"github.com/hashicorp/nomad/nomad/structs/config"
+
+	syslog "github.com/RackSec/srslog"
 
 	dstructs "github.com/hashicorp/nomad/client/driver/structs"
 	cstructs "github.com/hashicorp/nomad/client/structs"
@@ -38,6 +40,11 @@ const (
 	// tree for finding out the pids that the executor and it's child processes
 	// have forked
 	pidScanInterval = 5 * time.Second
+
+	// processOutputCloseTolerance is the length of time we will wait for the
+	// launched process to close its stdout/stderr before we force close it. If
+	// data is written after this tolerance, we will not capture it.
+	processOutputCloseTolerance = 2 * time.Second
 )
 
 var (
@@ -49,59 +56,34 @@ var (
 // Executor is the interface which allows a driver to launch and supervise
 // a process
 type Executor interface {
-	LaunchCmd(command *ExecCommand, ctx *ExecutorContext) (*ProcessState, error)
-	LaunchSyslogServer(ctx *ExecutorContext) (*SyslogServerState, error)
+	SetContext(ctx *ExecutorContext) error
+	LaunchCmd(command *ExecCommand) (*ProcessState, error)
+	LaunchSyslogServer() (*SyslogServerState, error)
 	Wait() (*ProcessState, error)
 	ShutDown() error
 	Exit() error
 	UpdateLogConfig(logConfig *structs.LogConfig) error
 	UpdateTask(task *structs.Task) error
-	SyncServices(ctx *ConsulContext) error
-	DeregisterServices() error
 	Version() (*ExecutorVersion, error)
 	Stats() (*cstructs.TaskResourceUsage, error)
-}
-
-// ConsulContext holds context to configure the Consul client and run checks
-type ConsulContext struct {
-	// ConsulConfig contains the configuration information for talking
-	// with this Nomad Agent's Consul Agent.
-	ConsulConfig *config.ConsulConfig
-
-	// ContainerID is the ID of the container
-	ContainerID string
-
-	// TLSCert is the cert which docker client uses while interactng with the docker
-	// daemon over TLS
-	TLSCert string
-
-	// TLSCa is the CA which the docker client uses while interacting with the docker
-	// daeemon over TLS
-	TLSCa string
-
-	// TLSKey is the TLS key which the docker client uses while interacting with
-	// the docker daemon
-	TLSKey string
-
-	// DockerEndpoint is the endpoint of the docker daemon
-	DockerEndpoint string
+	Signal(s os.Signal) error
+	Exec(deadline time.Time, cmd string, args []string) ([]byte, int, error)
 }
 
 // ExecutorContext holds context to configure the command user
 // wants to run and isolate it
 type ExecutorContext struct {
 	// TaskEnv holds information about the environment of a Task
-	TaskEnv *env.TaskEnvironment
-
-	// AllocDir is the handle to do operations on the alloc dir of
-	// the task
-	AllocDir *allocdir.AllocDir
+	TaskEnv *env.TaskEnv
 
 	// Task is the task whose executor is being launched
 	Task *structs.Task
 
-	// AllocID is the allocation id to which the task belongs
-	AllocID string
+	// TaskDir is the host path to the task's root
+	TaskDir string
+
+	// LogDir is the host path where logs should be written
+	LogDir string
 
 	// Driver is the name of the driver that invoked the executor
 	Driver string
@@ -124,6 +106,9 @@ type ExecCommand struct {
 	// Args is the args of the command that the user wants to run.
 	Args []string
 
+	// TaskKillSignal is an optional field which signal to kill the process
+	TaskKillSignal os.Signal
+
 	// FSIsolation determines whether the command would be run in a chroot.
 	FSIsolation bool
 
@@ -133,6 +118,11 @@ type ExecCommand struct {
 	// ResourceLimits determines whether resource limits are enforced by the
 	// executor.
 	ResourceLimits bool
+
+	// Cgroup marks whether we put the process in a cgroup. Setting this field
+	// doesn't enforce resource limits. To enforce limits, set ResourceLimits.
+	// Using the cgroup does allow more precise cleanup of processes.
+	BasicProcessCgroup bool
 }
 
 // ProcessState holds information about the state of a user process.
@@ -152,7 +142,7 @@ type nomadPid struct {
 	cpuStatsSys   *stats.CpuStats
 }
 
-// SyslogServerState holds the address and islation information of a launched
+// SyslogServerState holds the address and isolation information of a launched
 // syslog server
 type SyslogServerState struct {
 	IsolationConfig *dstructs.IsolationConfig
@@ -178,26 +168,19 @@ type UniversalExecutor struct {
 
 	pids                map[int]*nomadPid
 	pidLock             sync.RWMutex
-	taskDir             string
 	exitState           *ProcessState
 	processExited       chan interface{}
 	fsIsolationEnforced bool
 
-	lre         *logging.FileRotator
-	lro         *logging.FileRotator
+	lre         *logRotatorWrapper
+	lro         *logRotatorWrapper
 	rotatorLock sync.Mutex
 
-	shutdownCh chan struct{}
-
 	syslogServer *logging.SyslogServer
 	syslogChan   chan *logging.SyslogMessage
 
-	groups  *cgroupConfig.Cgroup
-	cgPaths map[string]string
-	cgLock  sync.Mutex
+	resConCtx resourceContainerContext
 
-	consulSyncer   *consul.Syncer
-	consulCtx      *ConsulContext
 	totalCpuStats  *stats.CpuStats
 	userCpuStats   *stats.CpuStats
 	systemCpuStats *stats.CpuStats
@@ -207,8 +190,7 @@ type UniversalExecutor struct {
 // NewExecutor returns an Executor
 func NewExecutor(logger *log.Logger) Executor {
 	if err := shelpers.Init(); err != nil {
-		logger.Printf("[FATAL] executor: unable to initialize stats: %v", err)
-		return nil
+		logger.Printf("[ERR] executor: unable to initialize stats: %v", err)
 	}
 
 	exec := &UniversalExecutor{
@@ -225,15 +207,26 @@ func NewExecutor(logger *log.Logger) Executor {
 
 // Version returns the api version of the executor
 func (e *UniversalExecutor) Version() (*ExecutorVersion, error) {
-	return &ExecutorVersion{Version: "1.0.0"}, nil
+	return &ExecutorVersion{Version: "1.1.0"}, nil
 }
 
-// LaunchCmd launches a process and returns it's state. It also configures an
-// applies isolation on certain platforms.
-func (e *UniversalExecutor) LaunchCmd(command *ExecCommand, ctx *ExecutorContext) (*ProcessState, error) {
-	e.logger.Printf("[DEBUG] executor: launching command %v %v", command.Cmd, strings.Join(command.Args, " "))
-
+// SetContext is used to set the executors context and should be the first call
+// after launching the executor.
+func (e *UniversalExecutor) SetContext(ctx *ExecutorContext) error {
 	e.ctx = ctx
+	return nil
+}
+
+// LaunchCmd launches the main process and returns its state. It also
+// configures an applies isolation on certain platforms.
+func (e *UniversalExecutor) LaunchCmd(command *ExecCommand) (*ProcessState, error) {
+	e.logger.Printf("[INFO] executor: launching command %v %v", command.Cmd, strings.Join(command.Args, " "))
+
+	// Ensure the context has been set first
+	if e.ctx == nil {
+		return nil, fmt.Errorf("SetContext must be called before launching a command")
+	}
+
 	e.command = command
 
 	// setting the user of the process
@@ -244,20 +237,23 @@ func (e *UniversalExecutor) LaunchCmd(command *ExecCommand, ctx *ExecutorContext
 		}
 	}
 
-	// configuring the task dir
-	if err := e.configureTaskDir(); err != nil {
+	// set the task dir as the working directory for the command
+	e.cmd.Dir = e.ctx.TaskDir
+
+	// start command in separate process group
+	if err := e.setNewProcessGroup(); err != nil {
 		return nil, err
 	}
 
-	e.ctx.TaskEnv.Build()
-	// configuring the chroot, cgroup and enters the plugin process in the
-	// chroot
+	// configuring the chroot, resource container, and start the plugin
+	// process in the chroot.
 	if err := e.configureIsolation(); err != nil {
 		return nil, err
 	}
-	// Apply ourselves into the cgroup. The executor MUST be in the cgroup
-	// before the user task is started, otherwise we are subject to a fork
-	// attack in which a process escapes isolation by immediately forking.
+	// Apply ourselves into the resource container. The executor MUST be in
+	// the resource container before the user task is started, otherwise we
+	// are subject to a fork attack in which a process escapes isolation by
+	// immediately forking.
 	if err := e.applyLimits(os.Getpid()); err != nil {
 		return nil, err
 	}
@@ -266,11 +262,11 @@ func (e *UniversalExecutor) LaunchCmd(command *ExecCommand, ctx *ExecutorContext
 	if err := e.configureLoggers(); err != nil {
 		return nil, err
 	}
-	e.cmd.Stdout = e.lro
-	e.cmd.Stderr = e.lre
+	e.cmd.Stdout = e.lro.processOutWriter
+	e.cmd.Stderr = e.lre.processOutWriter
 
 	// Look up the binary path and make it executable
-	absPath, err := e.lookupBin(ctx.TaskEnv.ReplaceEnv(command.Cmd))
+	absPath, err := e.lookupBin(e.ctx.TaskEnv.ReplaceEnv(command.Cmd))
 	if err != nil {
 		return nil, err
 	}
@@ -283,28 +279,78 @@ func (e *UniversalExecutor) LaunchCmd(command *ExecCommand, ctx *ExecutorContext
 
 	// Determine the path to run as it may have to be relative to the chroot.
 	if e.fsIsolationEnforced {
-		rel, err := filepath.Rel(e.taskDir, path)
+		rel, err := filepath.Rel(e.ctx.TaskDir, path)
 		if err != nil {
-			return nil, err
+			return nil, fmt.Errorf("failed to determine relative path base=%q target=%q: %v", e.ctx.TaskDir, path, err)
 		}
 		path = rel
 	}
 
 	// Set the commands arguments
 	e.cmd.Path = path
-	e.cmd.Args = append([]string{e.cmd.Path}, ctx.TaskEnv.ParseAndReplace(command.Args)...)
-	e.cmd.Env = ctx.TaskEnv.EnvList()
+	e.cmd.Args = append([]string{e.cmd.Path}, e.ctx.TaskEnv.ParseAndReplace(command.Args)...)
+	e.cmd.Env = e.ctx.TaskEnv.List()
 
 	// Start the process
 	if err := e.cmd.Start(); err != nil {
-		return nil, err
+		return nil, fmt.Errorf("failed to start command path=%q --- args=%q: %v", path, e.cmd.Args, err)
 	}
+
+	// Close the files. This is copied from the os/exec package.
+	e.lro.processOutWriter.Close()
+	e.lre.processOutWriter.Close()
+
 	go e.collectPids()
 	go e.wait()
-	ic := &dstructs.IsolationConfig{Cgroup: e.groups, CgroupPaths: e.cgPaths}
+	ic := e.resConCtx.getIsolationConfig()
 	return &ProcessState{Pid: e.cmd.Process.Pid, ExitCode: -1, IsolationConfig: ic, Time: time.Now()}, nil
 }
 
+// Exec a command inside a container for exec and java drivers.
+func (e *UniversalExecutor) Exec(deadline time.Time, name string, args []string) ([]byte, int, error) {
+	ctx, cancel := context.WithDeadline(context.Background(), deadline)
+	defer cancel()
+	return ExecScript(ctx, e.cmd.Dir, e.ctx.TaskEnv, e.cmd.SysProcAttr, name, args)
+}
+
+// ExecScript executes cmd with args and returns the output, exit code, and
+// error. Output is truncated to client/driver/structs.CheckBufSize
+func ExecScript(ctx context.Context, dir string, env *env.TaskEnv, attrs *syscall.SysProcAttr,
+	name string, args []string) ([]byte, int, error) {
+	name = env.ReplaceEnv(name)
+	cmd := exec.CommandContext(ctx, name, env.ParseAndReplace(args)...)
+
+	// Copy runtime environment from the main command
+	cmd.SysProcAttr = attrs
+	cmd.Dir = dir
+	cmd.Env = env.List()
+
+	// Capture output
+	buf, _ := circbuf.NewBuffer(int64(dstructs.CheckBufSize))
+	cmd.Stdout = buf
+	cmd.Stderr = buf
+
+	if err := cmd.Run(); err != nil {
+		exitErr, ok := err.(*exec.ExitError)
+		if !ok {
+			// Non-exit error, return it and let the caller treat
+			// it as a critical failure
+			return nil, 0, err
+		}
+
+		// Some kind of error happened; default to critical
+		exitCode := 2
+		if status, ok := exitErr.Sys().(syscall.WaitStatus); ok {
+			exitCode = status.ExitStatus()
+		}
+
+		// Don't return the exitError as the caller only needs the
+		// output and code.
+		return buf.Bytes(), exitCode, nil
+	}
+	return buf.Bytes(), 0, nil
+}
+
 // configureLoggers sets up the standard out/error file rotators
 func (e *UniversalExecutor) configureLoggers() error {
 	e.rotatorLock.Lock()
@@ -312,21 +358,31 @@ func (e *UniversalExecutor) configureLoggers() error {
 
 	logFileSize := int64(e.ctx.Task.LogConfig.MaxFileSizeMB * 1024 * 1024)
 	if e.lro == nil {
-		lro, err := logging.NewFileRotator(e.ctx.AllocDir.LogDir(), fmt.Sprintf("%v.stdout", e.ctx.Task.Name),
+		lro, err := logging.NewFileRotator(e.ctx.LogDir, fmt.Sprintf("%v.stdout", e.ctx.Task.Name),
 			e.ctx.Task.LogConfig.MaxFiles, logFileSize, e.logger)
+		if err != nil {
+			return fmt.Errorf("error creating new stdout log file for %q: %v", e.ctx.Task.Name, err)
+		}
+
+		r, err := newLogRotatorWrapper(e.logger, lro)
 		if err != nil {
 			return err
 		}
-		e.lro = lro
+		e.lro = r
 	}
 
 	if e.lre == nil {
-		lre, err := logging.NewFileRotator(e.ctx.AllocDir.LogDir(), fmt.Sprintf("%v.stderr", e.ctx.Task.Name),
+		lre, err := logging.NewFileRotator(e.ctx.LogDir, fmt.Sprintf("%v.stderr", e.ctx.Task.Name),
 			e.ctx.Task.LogConfig.MaxFiles, logFileSize, e.logger)
+		if err != nil {
+			return fmt.Errorf("error creating new stderr log file for %q: %v", e.ctx.Task.Name, err)
+		}
+
+		r, err := newLogRotatorWrapper(e.logger, lre)
 		if err != nil {
 			return err
 		}
-		e.lre = lre
+		e.lre = r
 	}
 	return nil
 }
@@ -344,14 +400,14 @@ func (e *UniversalExecutor) UpdateLogConfig(logConfig *structs.LogConfig) error
 	if e.lro == nil {
 		return fmt.Errorf("log rotator for stdout doesn't exist")
 	}
-	e.lro.MaxFiles = logConfig.MaxFiles
-	e.lro.FileSize = int64(logConfig.MaxFileSizeMB * 1024 * 1024)
+	e.lro.rotatorWriter.MaxFiles = logConfig.MaxFiles
+	e.lro.rotatorWriter.FileSize = int64(logConfig.MaxFileSizeMB * 1024 * 1024)
 
 	if e.lre == nil {
 		return fmt.Errorf("log rotator for stderr doesn't exist")
 	}
-	e.lre.MaxFiles = logConfig.MaxFiles
-	e.lre.FileSize = int64(logConfig.MaxFileSizeMB * 1024 * 1024)
+	e.lre.rotatorWriter.MaxFiles = logConfig.MaxFiles
+	e.lre.rotatorWriter.FileSize = int64(logConfig.MaxFileSizeMB * 1024 * 1024)
 	return nil
 }
 
@@ -359,41 +415,30 @@ func (e *UniversalExecutor) UpdateTask(task *structs.Task) error {
 	e.ctx.Task = task
 
 	// Updating Log Config
-	fileSize := int64(task.LogConfig.MaxFileSizeMB * 1024 * 1024)
-	e.lro.MaxFiles = task.LogConfig.MaxFiles
-	e.lro.FileSize = fileSize
-	e.lre.MaxFiles = task.LogConfig.MaxFiles
-	e.lre.FileSize = fileSize
-
-	// Re-syncing task with Consul agent
-	if e.consulSyncer != nil {
-		e.interpolateServices(e.ctx.Task)
-		domain := consul.NewExecutorDomain(e.ctx.AllocID, task.Name)
-		serviceMap := generateServiceKeys(e.ctx.AllocID, task.Services)
-		e.consulSyncer.SetServices(domain, serviceMap)
+	e.rotatorLock.Lock()
+	if e.lro != nil && e.lre != nil {
+		fileSize := int64(task.LogConfig.MaxFileSizeMB * 1024 * 1024)
+		e.lro.rotatorWriter.MaxFiles = task.LogConfig.MaxFiles
+		e.lro.rotatorWriter.FileSize = fileSize
+		e.lre.rotatorWriter.MaxFiles = task.LogConfig.MaxFiles
+		e.lre.rotatorWriter.FileSize = fileSize
 	}
+	e.rotatorLock.Unlock()
 	return nil
 }
 
-// generateServiceKeys takes a list of interpolated Nomad Services and returns a map
-// of ServiceKeys to Nomad Services.
-func generateServiceKeys(allocID string, services []*structs.Service) map[consul.ServiceKey]*structs.Service {
-	keys := make(map[consul.ServiceKey]*structs.Service, len(services))
-	for _, service := range services {
-		key := consul.GenerateServiceKey(service)
-		keys[key] = service
-	}
-	return keys
-}
-
 func (e *UniversalExecutor) wait() {
 	defer close(e.processExited)
 	err := e.cmd.Wait()
-	ic := &dstructs.IsolationConfig{Cgroup: e.groups, CgroupPaths: e.cgPaths}
+	ic := e.resConCtx.getIsolationConfig()
 	if err == nil {
 		e.exitState = &ProcessState{Pid: 0, ExitCode: 0, IsolationConfig: ic, Time: time.Now()}
 		return
 	}
+
+	e.lre.Close()
+	e.lro.Close()
+
 	exitCode := 1
 	var signal int
 	if exitErr, ok := err.(*exec.ExitError); ok {
@@ -413,7 +458,7 @@ func (e *UniversalExecutor) wait() {
 			}
 		}
 	} else {
-		e.logger.Printf("[DEBUG] executor: unexpected Wait() error type: %v", err)
+		e.logger.Printf("[WARN] executor: unexpected Cmd.Wait() error type: %v", err)
 	}
 
 	e.exitState = &ProcessState{Pid: 0, ExitCode: exitCode, Signal: signal, IsolationConfig: ic, Time: time.Now()}
@@ -423,20 +468,32 @@ var (
 	// finishedErr is the error message received when trying to kill and already
 	// exited process.
 	finishedErr = "os: process already finished"
+
+	// noSuchProcessErr is the error message received when trying to kill a non
+	// existing process (e.g. when killing a process group).
+	noSuchProcessErr = "no such process"
 )
 
-// Exit cleans up the alloc directory, destroys cgroups and kills the user
-// process
+// ClientCleanup is the cleanup routine that a Nomad Client uses to remove the
+// remnants of a child UniversalExecutor.
+func ClientCleanup(ic *dstructs.IsolationConfig, pid int) error {
+	return clientCleanup(ic, pid)
+}
+
+// Exit cleans up the alloc directory, destroys resource container and kills the
+// user process
 func (e *UniversalExecutor) Exit() error {
 	var merr multierror.Error
 	if e.syslogServer != nil {
 		e.syslogServer.Shutdown()
 	}
-	e.lre.Close()
-	e.lro.Close()
 
-	if e.consulSyncer != nil {
-		e.consulSyncer.Shutdown()
+	if e.lre != nil {
+		e.lre.Close()
+	}
+
+	if e.lro != nil {
+		e.lro.Close()
 	}
 
 	// If the executor did not launch a process, return.
@@ -444,28 +501,20 @@ func (e *UniversalExecutor) Exit() error {
 		return nil
 	}
 
-	// Prefer killing the process via cgroups.
-	if e.cmd.Process != nil && !e.command.ResourceLimits {
+	// Prefer killing the process via the resource container.
+	if e.cmd.Process != nil && !(e.command.ResourceLimits || e.command.BasicProcessCgroup) {
 		proc, err := os.FindProcess(e.cmd.Process.Pid)
 		if err != nil {
 			e.logger.Printf("[ERR] executor: can't find process with pid: %v, err: %v",
 				e.cmd.Process.Pid, err)
-		} else if err := proc.Kill(); err != nil && err.Error() != finishedErr {
+		} else if err := e.cleanupChildProcesses(proc); err != nil && err.Error() != finishedErr {
 			merr.Errors = append(merr.Errors,
 				fmt.Errorf("can't kill process with pid: %v, err: %v", e.cmd.Process.Pid, err))
 		}
 	}
 
-	if e.command.ResourceLimits {
-		e.cgLock.Lock()
-		if err := DestroyCgroup(e.groups, e.cgPaths, os.Getpid()); err != nil {
-			merr.Errors = append(merr.Errors, err)
-		}
-		e.cgLock.Unlock()
-	}
-
-	if e.command.FSIsolation {
-		if err := e.removeChrootMounts(); err != nil {
+	if e.command.ResourceLimits || e.command.BasicProcessCgroup {
+		if err := e.resConCtx.executorCleanup(); err != nil {
 			merr.Errors = append(merr.Errors, err)
 		}
 	}
@@ -481,48 +530,7 @@ func (e *UniversalExecutor) ShutDown() error {
 	if err != nil {
 		return fmt.Errorf("executor.shutdown failed to find process: %v", err)
 	}
-	if runtime.GOOS == "windows" {
-		if err := proc.Kill(); err != nil && err.Error() != finishedErr {
-			return err
-		}
-		return nil
-	}
-	if err = proc.Signal(os.Interrupt); err != nil && err.Error() != finishedErr {
-		return fmt.Errorf("executor.shutdown error: %v", err)
-	}
-	return nil
-}
-
-// SyncServices syncs the services of the task that the executor is running with
-// Consul
-func (e *UniversalExecutor) SyncServices(ctx *ConsulContext) error {
-	e.logger.Printf("[INFO] executor: registering services")
-	e.consulCtx = ctx
-	if e.consulSyncer == nil {
-		cs, err := consul.NewSyncer(ctx.ConsulConfig, e.shutdownCh, e.logger)
-		if err != nil {
-			return err
-		}
-		e.consulSyncer = cs
-		go e.consulSyncer.Run()
-	}
-	e.interpolateServices(e.ctx.Task)
-	e.consulSyncer.SetDelegatedChecks(e.createCheckMap(), e.createCheck)
-	e.consulSyncer.SetAddrFinder(e.ctx.Task.FindHostAndPortFor)
-	domain := consul.NewExecutorDomain(e.ctx.AllocID, e.ctx.Task.Name)
-	serviceMap := generateServiceKeys(e.ctx.AllocID, e.ctx.Task.Services)
-	e.consulSyncer.SetServices(domain, serviceMap)
-	return nil
-}
-
-// DeregisterServices removes the services of the task that the executor is
-// running from Consul
-func (e *UniversalExecutor) DeregisterServices() error {
-	e.logger.Printf("[INFO] executor: de-registering services and shutting down consul service")
-	if e.consulSyncer != nil {
-		return e.consulSyncer.Shutdown()
-	}
-	return nil
+	return e.shutdownProcess(proc)
 }
 
 // pidStats returns the resource usage stats per pid
@@ -537,7 +545,7 @@ func (e *UniversalExecutor) pidStats() (map[string]*cstructs.ResourceUsage, erro
 	for pid, np := range pids {
 		p, err := process.NewProcess(int32(pid))
 		if err != nil {
-			e.logger.Printf("[DEBUG] executor: unable to create new process with pid: %v", pid)
+			e.logger.Printf("[TRACE] executor: unable to create new process with pid: %v", pid)
 			continue
 		}
 		ms := &cstructs.MemoryStats{}
@@ -562,29 +570,18 @@ func (e *UniversalExecutor) pidStats() (map[string]*cstructs.ResourceUsage, erro
 	return stats, nil
 }
 
-// configureTaskDir sets the task dir in the executor
-func (e *UniversalExecutor) configureTaskDir() error {
-	taskDir, ok := e.ctx.AllocDir.TaskDirs[e.ctx.Task.Name]
-	e.taskDir = taskDir
-	if !ok {
-		return fmt.Errorf("couldn't find task directory for task %v", e.ctx.Task.Name)
-	}
-	e.cmd.Dir = taskDir
-	return nil
-}
-
 // lookupBin looks for path to the binary to run by looking for the binary in
 // the following locations, in-order: task/local/, task/, based on host $PATH.
 // The return path is absolute.
 func (e *UniversalExecutor) lookupBin(bin string) (string, error) {
 	// Check in the local directory
-	local := filepath.Join(e.taskDir, allocdir.TaskLocal, bin)
+	local := filepath.Join(e.ctx.TaskDir, allocdir.TaskLocal, bin)
 	if _, err := os.Stat(local); err == nil {
 		return local, nil
 	}
 
 	// Check at the root of the task's directory
-	root := filepath.Join(e.taskDir, bin)
+	root := filepath.Join(e.ctx.TaskDir, bin)
 	if _, err := os.Stat(root); err == nil {
 		return root, nil
 	}
@@ -667,64 +664,6 @@ func (e *UniversalExecutor) listenerUnix() (net.Listener, error) {
 	return net.Listen("unix", path)
 }
 
-// createCheckMap creates a map of checks that the executor will handle on it's
-// own
-func (e *UniversalExecutor) createCheckMap() map[string]struct{} {
-	checks := map[string]struct{}{
-		"script": struct{}{},
-	}
-	return checks
-}
-
-// createCheck creates NomadCheck from a ServiceCheck
-func (e *UniversalExecutor) createCheck(check *structs.ServiceCheck, checkID string) (consul.Check, error) {
-	if check.Type == structs.ServiceCheckScript && e.ctx.Driver == "docker" {
-		return &DockerScriptCheck{
-			id:          checkID,
-			interval:    check.Interval,
-			timeout:     check.Timeout,
-			containerID: e.consulCtx.ContainerID,
-			logger:      e.logger,
-			cmd:         check.Command,
-			args:        check.Args,
-		}, nil
-	}
-
-	if check.Type == structs.ServiceCheckScript && (e.ctx.Driver == "exec" ||
-		e.ctx.Driver == "raw_exec" || e.ctx.Driver == "java") {
-		return &ExecScriptCheck{
-			id:          checkID,
-			interval:    check.Interval,
-			timeout:     check.Timeout,
-			cmd:         check.Command,
-			args:        check.Args,
-			taskDir:     e.taskDir,
-			FSIsolation: e.command.FSIsolation,
-		}, nil
-
-	}
-	return nil, fmt.Errorf("couldn't create check for %v", check.Name)
-}
-
-// interpolateServices interpolates tags in a service and checks with values from the
-// task's environment.
-func (e *UniversalExecutor) interpolateServices(task *structs.Task) {
-	e.ctx.TaskEnv.Build()
-	for _, service := range task.Services {
-		for _, check := range service.Checks {
-			if check.Type == structs.ServiceCheckScript {
-				check.Name = e.ctx.TaskEnv.ReplaceEnv(check.Name)
-				check.Command = e.ctx.TaskEnv.ReplaceEnv(check.Command)
-				check.Args = e.ctx.TaskEnv.ParseAndReplace(check.Args)
-				check.Path = e.ctx.TaskEnv.ReplaceEnv(check.Path)
-				check.Protocol = e.ctx.TaskEnv.ReplaceEnv(check.Protocol)
-			}
-		}
-		service.Name = e.ctx.TaskEnv.ReplaceEnv(service.Name)
-		service.Tags = e.ctx.TaskEnv.ParseAndReplace(service.Tags)
-	}
-}
-
 // collectPids collects the pids of the child processes that the executor is
 // running every 5 seconds
 func (e *UniversalExecutor) collectPids() {
@@ -767,25 +706,26 @@ func (e *UniversalExecutor) scanPids(parentPid int, allPids []ps.Process) (map[i
 	processFamily := make(map[int]struct{})
 	processFamily[parentPid] = struct{}{}
 
-	// A buffer for holding pids which haven't matched with any parent pid
-	var pidsRemaining []ps.Process
+	// A mapping of pids to their parent pids. It is used to build the process
+	// tree of the executing task
+	pidsRemaining := make(map[int]int, len(allPids))
+	for _, pid := range allPids {
+		pidsRemaining[pid.Pid()] = pid.PPid()
+	}
+
 	for {
 		// flag to indicate if we have found a match
 		foundNewPid := false
 
-		for _, pid := range allPids {
-			_, childPid := processFamily[pid.PPid()]
+		for pid, ppid := range pidsRemaining {
+			_, childPid := processFamily[ppid]
 
 			// checking if the pid is a child of any of the parents
 			if childPid {
-				processFamily[pid.Pid()] = struct{}{}
+				processFamily[pid] = struct{}{}
+				delete(pidsRemaining, pid)
 				foundNewPid = true
-			} else {
-				// if it is not, then we add the pid to the buffer
-				pidsRemaining = append(pidsRemaining, pid)
 			}
-			// scan only the pids which are left in the buffer
-			allPids = pidsRemaining
 		}
 
 		// not scanning anymore if we couldn't find a single match
@@ -793,6 +733,7 @@ func (e *UniversalExecutor) scanPids(parentPid int, allPids []ps.Process) (map[i
 			break
 		}
 	}
+
 	res := make(map[int]*nomadPid)
 	for pid := range processFamily {
 		np := nomadPid{
@@ -848,3 +789,139 @@ func (e *UniversalExecutor) aggregatedResourceUsage(pidStats map[string]*cstruct
 		Pids:          pidStats,
 	}
 }
+
+// Signal sends the passed signal to the task
+func (e *UniversalExecutor) Signal(s os.Signal) error {
+	if e.cmd.Process == nil {
+		return fmt.Errorf("Task not yet run")
+	}
+
+	e.logger.Printf("[DEBUG] executor: sending signal %s to PID %d", s, e.cmd.Process.Pid)
+	err := e.cmd.Process.Signal(s)
+	if err != nil {
+		e.logger.Printf("[ERR] executor: sending signal %v failed: %v", s, err)
+		return err
+	}
+
+	return nil
+}
+
+func (e *UniversalExecutor) LaunchSyslogServer() (*SyslogServerState, error) {
+	// Ensure the context has been set first
+	if e.ctx == nil {
+		return nil, fmt.Errorf("SetContext must be called before launching the Syslog Server")
+	}
+
+	e.syslogChan = make(chan *logging.SyslogMessage, 2048)
+	l, err := e.getListener(e.ctx.PortLowerBound, e.ctx.PortUpperBound)
+	if err != nil {
+		return nil, err
+	}
+	e.logger.Printf("[DEBUG] syslog-server: launching syslog server on addr: %v", l.Addr().String())
+	if err := e.configureLoggers(); err != nil {
+		return nil, err
+	}
+
+	e.syslogServer = logging.NewSyslogServer(l, e.syslogChan, e.logger)
+	go e.syslogServer.Start()
+	go e.collectLogs(e.lre.rotatorWriter, e.lro.rotatorWriter)
+	syslogAddr := fmt.Sprintf("%s://%s", l.Addr().Network(), l.Addr().String())
+	return &SyslogServerState{Addr: syslogAddr}, nil
+}
+
+func (e *UniversalExecutor) collectLogs(we io.Writer, wo io.Writer) {
+	for logParts := range e.syslogChan {
+		// If the severity of the log line is err then we write to stderr
+		// otherwise all messages go to stdout
+		if logParts.Severity == syslog.LOG_ERR {
+			we.Write(logParts.Message)
+			we.Write([]byte{'\n'})
+		} else {
+			wo.Write(logParts.Message)
+			wo.Write([]byte{'\n'})
+		}
+	}
+}
+
+// logRotatorWrapper wraps our log rotator and exposes a pipe that can feed the
+// log rotator data. The processOutWriter should be attached to the process and
+// data will be copied from the reader to the rotator.
+type logRotatorWrapper struct {
+	processOutWriter  *os.File
+	processOutReader  *os.File
+	rotatorWriter     *logging.FileRotator
+	hasFinishedCopied chan struct{}
+	logger            *log.Logger
+}
+
+// newLogRotatorWrapper takes a rotator and returns a wrapper that has the
+// processOutWriter to attach to the processes stdout or stderr.
+func newLogRotatorWrapper(logger *log.Logger, rotator *logging.FileRotator) (*logRotatorWrapper, error) {
+	r, w, err := os.Pipe()
+	if err != nil {
+		return nil, fmt.Errorf("failed to create os.Pipe for extracting logs: %v", err)
+	}
+
+	wrap := &logRotatorWrapper{
+		processOutWriter:  w,
+		processOutReader:  r,
+		rotatorWriter:     rotator,
+		hasFinishedCopied: make(chan struct{}),
+		logger:            logger,
+	}
+	wrap.start()
+	return wrap, nil
+}
+
+// start starts a go-routine that copies from the pipe into the rotator. This is
+// called by the constructor and not the user of the wrapper.
+func (l *logRotatorWrapper) start() {
+	go func() {
+		defer close(l.hasFinishedCopied)
+		_, err := io.Copy(l.rotatorWriter, l.processOutReader)
+		if err != nil {
+			// Close reader to propagate io error across pipe.
+			// Note that this may block until the process exits on
+			// Windows due to
+			// https://github.com/PowerShell/PowerShell/issues/4254
+			// or similar issues. Since this is already running in
+			// a goroutine its safe to block until the process is
+			// force-killed.
+			l.processOutReader.Close()
+		}
+	}()
+	return
+}
+
+// Close closes the rotator and the process writer to ensure that the Wait
+// command exits.
+func (l *logRotatorWrapper) Close() {
+	// Wait up to the close tolerance before we force close
+	select {
+	case <-l.hasFinishedCopied:
+	case <-time.After(processOutputCloseTolerance):
+	}
+
+	// Closing the read side of a pipe may block on Windows if the process
+	// is being debugged as in:
+	// https://github.com/PowerShell/PowerShell/issues/4254
+	// The pipe will be closed and cleaned up when the process exits.
+	closeDone := make(chan struct{})
+	go func() {
+		defer close(closeDone)
+		err := l.processOutReader.Close()
+		if err != nil && !strings.Contains(err.Error(), "file already closed") {
+			l.logger.Printf("[WARN] executor: error closing read-side of process output pipe: %v", err)
+		}
+
+	}()
+
+	select {
+	case <-closeDone:
+	case <-time.After(processOutputCloseTolerance):
+		l.logger.Printf("[WARN] executor: timed out waiting for read-side of process output pipe to close")
+	}
+
+	l.rotatorWriter.Close()
+	return
+}
