@@ -5,11 +5,14 @@ package fingerprint
 import (
 	"fmt"
 
-	client "github.com/hashicorp/nomad/client/config"
-	"github.com/hashicorp/nomad/nomad/structs"
+	cstructs "github.com/hashicorp/nomad/client/structs"
 	"github.com/opencontainers/runc/libcontainer/cgroups"
 )
 
+const (
+	cgroupAvailable = "available"
+)
+
 // FindCgroupMountpointDir is used to find the cgroup mount point on a Linux
 // system.
 func FindCgroupMountpointDir() (string, error) {
@@ -27,31 +30,32 @@ func FindCgroupMountpointDir() (string, error) {
 	return mount, nil
 }
 
-// Fingerprint tries to find a valid cgroup moint point
-func (f *CGroupFingerprint) Fingerprint(cfg *client.Config, node *structs.Node) (bool, error) {
+// Fingerprint tries to find a valid cgroup mount point
+func (f *CGroupFingerprint) Fingerprint(req *cstructs.FingerprintRequest, resp *cstructs.FingerprintResponse) error {
 	mount, err := f.mountPointDetector.MountPoint()
 	if err != nil {
-		f.clearCGroupAttributes(node)
-		return false, fmt.Errorf("Failed to discover cgroup mount point: %s", err)
+		f.clearCGroupAttributes(resp)
+		return fmt.Errorf("Failed to discover cgroup mount point: %s", err)
 	}
 
 	// Check if a cgroup mount point was found
 	if mount == "" {
-		// Clear any attributes from the previous fingerprint.
-		f.clearCGroupAttributes(node)
+
+		f.clearCGroupAttributes(resp)
 
 		if f.lastState == cgroupAvailable {
 			f.logger.Printf("[INFO] fingerprint.cgroups: cgroups are unavailable")
 		}
 		f.lastState = cgroupUnavailable
-		return true, nil
+		return nil
 	}
 
-	node.Attributes["unique.cgroup.mountpoint"] = mount
+	resp.AddAttribute("unique.cgroup.mountpoint", mount)
+	resp.Detected = true
 
 	if f.lastState == cgroupUnavailable {
 		f.logger.Printf("[INFO] fingerprint.cgroups: cgroups are available")
 	}
 	f.lastState = cgroupAvailable
-	return true, nil
+	return nil
 }
