@@ -4,6 +4,7 @@ import (
 	"testing"
 
 	"github.com/hashicorp/nomad/client/config"
+	cstructs "github.com/hashicorp/nomad/client/structs"
 	"github.com/hashicorp/nomad/nomad/structs"
 )
 
@@ -12,16 +13,24 @@ func TestHostFingerprint(t *testing.T) {
 	node := &structs.Node{
 		Attributes: make(map[string]string),
 	}
-	ok, err := f.Fingerprint(&config.Config{}, node)
+
+	request := &cstructs.FingerprintRequest{Config: &config.Config{}, Node: node}
+	var response cstructs.FingerprintResponse
+	err := f.Fingerprint(request, &response)
 	if err != nil {
 		t.Fatalf("err: %v", err)
 	}
-	if !ok {
-		t.Fatalf("should apply")
+
+	if !response.Detected {
+		t.Fatalf("expected response to be applicable")
+	}
+
+	if len(response.Attributes) == 0 {
+		t.Fatalf("should generate a diff of node attributes")
 	}
 
 	// Host info
 	for _, key := range []string{"os.name", "os.version", "unique.hostname", "kernel.name"} {
-		assertNodeAttributeContains(t, node, key)
+		assertNodeAttributeContains(t, response.Attributes, key)
 	}
 }
