@@ -13,17 +13,21 @@ func TestStorageFingerprint(t *testing.T) {
 		Attributes: make(map[string]string),
 	}
 
-	assertFingerprintOK(t, fp, node)
+	response := assertFingerprintOK(t, fp, node)
 
-	assertNodeAttributeContains(t, node, "unique.storage.volume")
-	assertNodeAttributeContains(t, node, "unique.storage.bytestotal")
-	assertNodeAttributeContains(t, node, "unique.storage.bytesfree")
+	if !response.Detected {
+		t.Fatalf("expected response to be applicable")
+	}
 
-	total, err := strconv.ParseInt(node.Attributes["unique.storage.bytestotal"], 10, 64)
+	assertNodeAttributeContains(t, response.Attributes, "unique.storage.volume")
+	assertNodeAttributeContains(t, response.Attributes, "unique.storage.bytestotal")
+	assertNodeAttributeContains(t, response.Attributes, "unique.storage.bytesfree")
+
+	total, err := strconv.ParseInt(response.Attributes["unique.storage.bytestotal"], 10, 64)
 	if err != nil {
 		t.Fatalf("Failed to parse unique.storage.bytestotal: %s", err)
 	}
-	free, err := strconv.ParseInt(node.Attributes["unique.storage.bytesfree"], 10, 64)
+	free, err := strconv.ParseInt(response.Attributes["unique.storage.bytesfree"], 10, 64)
 	if err != nil {
 		t.Fatalf("Failed to parse unique.storage.bytesfree: %s", err)
 	}
@@ -32,10 +36,10 @@ func TestStorageFingerprint(t *testing.T) {
 		t.Fatalf("unique.storage.bytesfree %d is larger than unique.storage.bytestotal %d", free, total)
 	}
 
-	if node.Resources == nil {
+	if response.Resources == nil {
 		t.Fatalf("Node Resources was nil")
 	}
-	if node.Resources.DiskMB == 0 {
+	if response.Resources.DiskMB == 0 {
 		t.Errorf("Expected node.Resources.DiskMB to be non-zero")
 	}
 }
