@@ -8,8 +8,9 @@ import (
 )
 
 func TestAgentCheckCommand_ServerHealth(t *testing.T) {
-	srv, _, url := testServer(t, nil)
-	defer srv.Stop()
+	t.Parallel()
+	srv, _, url := testServer(t, false, nil)
+	defer srv.Shutdown()
 
 	ui := new(cli.MockUi)
 	cmd := &AgentCheckCommand{Meta: Meta{Ui: ui}}
