@@ -4,13 +4,16 @@ import (
 	"fmt"
 	"math"
 	"sort"
+	"strconv"
 	"strings"
 	"time"
 
-	"github.com/dustin/go-humanize"
-	"github.com/mitchellh/colorstring"
+	humanize "github.com/dustin/go-humanize"
+	"github.com/posener/complete"
 
 	"github.com/hashicorp/nomad/api"
+	"github.com/hashicorp/nomad/api/contexts"
+	"github.com/hashicorp/nomad/helper"
 )
 
 const (
@@ -23,18 +26,19 @@ const (
 
 type NodeStatusCommand struct {
 	Meta
-	color       *colorstring.Colorize
 	length      int
 	short       bool
 	verbose     bool
 	list_allocs bool
 	self        bool
 	stats       bool
+	json        bool
+	tmpl        string
 }
 
 func (c *NodeStatusCommand) Help() string {
 	helpText := `
-Usage: nomad node-status [options] <node>
+Usage: nomad node status [options] <node>
 
   Display status information about a given node. The list of nodes
   returned includes only nodes which jobs may be scheduled to, and
@@ -54,7 +58,7 @@ Node Status Options:
   -self
     Query the status of the local node.
 
-  -stats 
+  -stats
     Display detailed resource usage statistics.
 
   -allocs
@@ -66,6 +70,12 @@ Node Status Options:
 
   -verbose
     Display full information.
+
+  -json
+    Output the node in its JSON format.
+
+  -t
+    Format and display node using a Go template.
 `
 	return strings.TrimSpace(helpText)
 }
@@ -74,15 +84,47 @@ func (c *NodeStatusCommand) Synopsis() string {
 	return "Display status information about nodes"
 }
 
+func (c *NodeStatusCommand) AutocompleteFlags() complete.Flags {
+	return mergeAutocompleteFlags(c.Meta.AutocompleteFlags(FlagSetClient),
+		complete.Flags{
+			"-allocs":  complete.PredictNothing,
+			"-json":    complete.PredictNothing,
+			"-self":    complete.PredictNothing,
+			"-short":   complete.PredictNothing,
+			"-stats":   complete.PredictNothing,
+			"-t":       complete.PredictAnything,
+			"-verbose": complete.PredictNothing,
+		})
+}
+
+func (c *NodeStatusCommand) AutocompleteArgs() complete.Predictor {
+	return complete.PredictFunc(func(a complete.Args) []string {
+		client, err := c.Meta.Client()
+		if err != nil {
+			return nil
+		}
+
+		resp, _, err := client.Search().PrefixSearch(a.Last, contexts.Nodes, nil)
+		if err != nil {
+			return []string{}
+		}
+		return resp.Matches[contexts.Nodes]
+	})
+}
+
+func (c *NodeStatusCommand) Name() string { return "node-status" }
+
 func (c *NodeStatusCommand) Run(args []string) int {
 
-	flags := c.Meta.FlagSet("node-status", FlagSetClient)
+	flags := c.Meta.FlagSet(c.Name(), FlagSetClient)
 	flags.Usage = func() { c.Ui.Output(c.Help()) }
 	flags.BoolVar(&c.short, "short", false, "")
 	flags.BoolVar(&c.verbose, "verbose", false, "")
 	flags.BoolVar(&c.list_allocs, "allocs", false, "")
 	flags.BoolVar(&c.self, "self", false, "")
 	flags.BoolVar(&c.stats, "stats", false, "")
+	flags.BoolVar(&c.json, "json", false, "")
+	flags.StringVar(&c.tmpl, "t", "", "")
 
 	if err := flags.Parse(args); err != nil {
 		return 1
@@ -91,7 +133,8 @@ func (c *NodeStatusCommand) Run(args []string) int {
 	// Check that we got either a single node or none
 	args = flags.Args()
 	if len(args) > 1 {
-		c.Ui.Error(c.Help())
+		c.Ui.Error("This command takes either one or no arguments")
+		c.Ui.Error(commandErrorText(c))
 		return 1
 	}
 
@@ -110,6 +153,7 @@ func (c *NodeStatusCommand) Run(args []string) int {
 
 	// Use list mode if no node name was provided
 	if len(args) == 0 && !c.self {
+
 		// Query the node info
 		nodes, _, err := client.Nodes().List(nil)
 		if err != nil {
@@ -117,6 +161,18 @@ func (c *NodeStatusCommand) Run(args []string) int {
 			return 1
 		}
 
+		// If output format is specified, format and output the node data list
+		if c.json || len(c.tmpl) > 0 {
+			out, err := Format(c.json, c.tmpl, nodes)
+			if err != nil {
+				c.Ui.Error(err.Error())
+				return 1
+			}
+
+			c.Ui.Output(out)
+			return 0
+		}
+
 		// Return nothing if no nodes found
 		if len(nodes) == 0 {
 			return 0
@@ -124,34 +180,42 @@ func (c *NodeStatusCommand) Run(args []string) int {
 
 		// Format the nodes list
 		out := make([]string, len(nodes)+1)
-		if c.list_allocs {
-			out[0] = "ID|DC|Name|Class|Drain|Status|Running Allocs"
-		} else {
-			out[0] = "ID|DC|Name|Class|Drain|Status"
+
+		out[0] = "ID|DC|Name|Class|"
+
+		if c.verbose {
+			out[0] += "Address|Version|"
 		}
+
+		out[0] += "Drain|Eligibility|Status"
+
+		if c.list_allocs {
+			out[0] += "|Running Allocs"
+		}
+
 		for i, node := range nodes {
+			out[i+1] = fmt.Sprintf("%s|%s|%s|%s",
+				limit(node.ID, c.length),
+				node.Datacenter,
+				node.Name,
+				node.NodeClass)
+			if c.verbose {
+				out[i+1] += fmt.Sprintf("|%s|%s",
+					node.Address, node.Version)
+			}
+			out[i+1] += fmt.Sprintf("|%v|%s|%s",
+				node.Drain,
+				node.SchedulingEligibility,
+				node.Status)
+
 			if c.list_allocs {
 				numAllocs, err := getRunningAllocs(client, node.ID)
 				if err != nil {
 					c.Ui.Error(fmt.Sprintf("Error querying node allocations: %s", err))
 					return 1
 				}
-				out[i+1] = fmt.Sprintf("%s|%s|%s|%s|%v|%s|%v",
-					limit(node.ID, c.length),
-					node.Datacenter,
-					node.Name,
-					node.NodeClass,
-					node.Drain,
-					node.Status,
+				out[i+1] += fmt.Sprintf("|%v",
 					len(numAllocs))
-			} else {
-				out[i+1] = fmt.Sprintf("%s|%s|%s|%s|%v|%s",
-					limit(node.ID, c.length),
-					node.Datacenter,
-					node.Name,
-					node.NodeClass,
-					node.Drain,
-					node.Status)
 			}
 		}
 
@@ -161,7 +225,7 @@ func (c *NodeStatusCommand) Run(args []string) int {
 	}
 
 	// Query the specific node
-	nodeID := ""
+	var nodeID string
 	if !c.self {
 		nodeID = args[0]
 	} else {
@@ -175,12 +239,8 @@ func (c *NodeStatusCommand) Run(args []string) int {
 		c.Ui.Error(fmt.Sprintf("Identifier must contain at least two characters."))
 		return 1
 	}
-	if len(nodeID)%2 == 1 {
-		// Identifiers must be of even length, so we strip off the last byte
-		// to provide a consistent user experience.
-		nodeID = nodeID[:len(nodeID)-1]
-	}
 
+	nodeID = sanitizeUUIDPrefix(nodeID)
 	nodes, _, err := client.Nodes().PrefixList(nodeID)
 	if err != nil {
 		c.Ui.Error(fmt.Sprintf("Error querying node info: %s", err))
@@ -192,23 +252,12 @@ func (c *NodeStatusCommand) Run(args []string) int {
 		return 1
 	}
 	if len(nodes) > 1 {
-		// Format the nodes list that matches the prefix so that the user
-		// can create a more specific request
-		out := make([]string, len(nodes)+1)
-		out[0] = "ID|DC|Name|Class|Drain|Status"
-		for i, node := range nodes {
-			out[i+1] = fmt.Sprintf("%s|%s|%s|%s|%v|%s",
-				limit(node.ID, c.length),
-				node.Datacenter,
-				node.Name,
-				node.NodeClass,
-				node.Drain,
-				node.Status)
-		}
 		// Dump the output
-		c.Ui.Output(fmt.Sprintf("Prefix matched multiple nodes\n\n%s", formatList(out)))
-		return 0
+		c.Ui.Error(fmt.Sprintf("Prefix matched multiple nodes\n\n%s",
+			formatNodeStubList(nodes, c.verbose)))
+		return 1
 	}
+
 	// Prefix lookup matched a single node
 	node, _, err := client.Nodes().Info(nodes[0].ID, nil)
 	if err != nil {
@@ -216,33 +265,104 @@ func (c *NodeStatusCommand) Run(args []string) int {
 		return 1
 	}
 
+	// If output format is specified, format and output the data
+	if c.json || len(c.tmpl) > 0 {
+		out, err := Format(c.json, c.tmpl, node)
+		if err != nil {
+			c.Ui.Error(err.Error())
+			return 1
+		}
+
+		c.Ui.Output(out)
+		return 0
+	}
+
 	return c.formatNode(client, node)
 }
 
-func (c *NodeStatusCommand) formatNode(client *api.Client, node *api.Node) int {
-	// Get the host stats
-	hostStats, nodeStatsErr := client.Nodes().Stats(node.ID, nil)
-	if nodeStatsErr != nil {
-		c.Ui.Output("")
-		c.Ui.Error(fmt.Sprintf("error fetching node stats (HINT: ensure Client.Advertise.HTTP is set): %v", nodeStatsErr))
+func nodeDrivers(n *api.Node) []string {
+	var drivers []string
+	for k, v := range n.Attributes {
+		// driver.docker = 1
+		parts := strings.Split(k, ".")
+		if len(parts) != 2 {
+			continue
+		} else if parts[0] != "driver" {
+			continue
+		} else if v != "1" {
+			continue
+		}
+
+		drivers = append(drivers, parts[1])
 	}
 
+	sort.Strings(drivers)
+	return drivers
+}
+
+func formatDrain(n *api.Node) string {
+	if n.DrainStrategy != nil {
+		b := new(strings.Builder)
+		b.WriteString("true")
+		if n.DrainStrategy.DrainSpec.Deadline.Nanoseconds() < 0 {
+			b.WriteString("; force drain")
+		} else if n.DrainStrategy.ForceDeadline.IsZero() {
+			b.WriteString("; no deadline")
+		} else {
+			fmt.Fprintf(b, "; %s deadline", formatTime(n.DrainStrategy.ForceDeadline))
+		}
+
+		if n.DrainStrategy.IgnoreSystemJobs {
+			b.WriteString("; ignoring system jobs")
+		}
+		return b.String()
+	}
+
+	return strconv.FormatBool(n.Drain)
+}
+
+func (c *NodeStatusCommand) formatNode(client *api.Client, node *api.Node) int {
 	// Format the header output
 	basic := []string{
 		fmt.Sprintf("ID|%s", limit(node.ID, c.length)),
 		fmt.Sprintf("Name|%s", node.Name),
 		fmt.Sprintf("Class|%s", node.NodeClass),
 		fmt.Sprintf("DC|%s", node.Datacenter),
-		fmt.Sprintf("Drain|%v", node.Drain),
+		fmt.Sprintf("Drain|%v", formatDrain(node)),
+		fmt.Sprintf("Eligibility|%s", node.SchedulingEligibility),
 		fmt.Sprintf("Status|%s", node.Status),
 	}
-	if hostStats != nil {
-		uptime := time.Duration(hostStats.Uptime * uint64(time.Second))
-		basic = append(basic, fmt.Sprintf("Uptime|%s", uptime.String()))
-	}
-	c.Ui.Output(c.Colorize().Color(formatKV(basic)))
 
-	if !c.short {
+	if c.short {
+		basic = append(basic, fmt.Sprintf("Drivers|%s", strings.Join(nodeDrivers(node), ",")))
+		c.Ui.Output(c.Colorize().Color(formatKV(basic)))
+	} else {
+		// Get the host stats
+		hostStats, nodeStatsErr := client.Nodes().Stats(node.ID, nil)
+		if nodeStatsErr != nil {
+			c.Ui.Output("")
+			c.Ui.Error(fmt.Sprintf("error fetching node stats: %v", nodeStatsErr))
+		}
+		if hostStats != nil {
+			uptime := time.Duration(hostStats.Uptime * uint64(time.Second))
+			basic = append(basic, fmt.Sprintf("Uptime|%s", uptime.String()))
+		}
+
+		// Emit the driver info
+		if !c.verbose {
+			driverStatus := fmt.Sprintf("Driver Status| %s", c.outputTruncatedNodeDriverInfo(node))
+			basic = append(basic, driverStatus)
+		}
+
+		c.Ui.Output(c.Colorize().Color(formatKV(basic)))
+
+		if c.verbose {
+			c.outputNodeDriverInfo(node)
+		}
+
+		// Emit node events
+		c.outputNodeStatusEvents(node)
+
 		// Get list of running allocations on the node
 		runningAllocs, err := getRunningAllocs(client, node.ID)
 		if err != nil {
@@ -263,7 +383,7 @@ func (c *NodeStatusCommand) formatNode(client *api.Client, node *api.Node) int {
 		hostResources, err := getHostResources(hostStats, node)
 		if err != nil {
 			c.Ui.Output("")
-			c.Ui.Error(fmt.Sprintf("error fetching node stats (HINT: ensure Client.Advertise.HTTP is set): %v", err))
+			c.Ui.Error(fmt.Sprintf("error fetching node stats: %v", err))
 		}
 		if err == nil {
 			c.Ui.Output(c.Colorize().Color("\n[bold]Host Resource Utilization[reset]"))
@@ -280,24 +400,109 @@ func (c *NodeStatusCommand) formatNode(client *api.Client, node *api.Node) int {
 		}
 	}
 
-	allocs, err := getAllocs(client, node, c.length)
+	nodeAllocs, _, err := client.Nodes().Allocations(node.ID, nil)
 	if err != nil {
 		c.Ui.Error(fmt.Sprintf("Error querying node allocations: %s", err))
 		return 1
 	}
 
-	if len(allocs) > 1 {
-		c.Ui.Output(c.Colorize().Color("\n[bold]Allocations[reset]"))
-		c.Ui.Output(formatList(allocs))
-	}
+	c.Ui.Output(c.Colorize().Color("\n[bold]Allocations[reset]"))
+	c.Ui.Output(formatAllocList(nodeAllocs, c.verbose, c.length))
 
 	if c.verbose {
 		c.formatAttributes(node)
+		c.formatMeta(node)
 	}
 	return 0
 
 }
 
+func (c *NodeStatusCommand) outputTruncatedNodeDriverInfo(node *api.Node) string {
+	drivers := make([]string, 0, len(node.Drivers))
+
+	for driverName, driverInfo := range node.Drivers {
+		if !driverInfo.Detected {
+			continue
+		}
+
+		if !driverInfo.Healthy {
+			drivers = append(drivers, fmt.Sprintf("%s (unhealthy)", driverName))
+		} else {
+			drivers = append(drivers, driverName)
+		}
+	}
+	sort.Strings(drivers)
+	return strings.Trim(strings.Join(drivers, ","), ", ")
+}
+
+func (c *NodeStatusCommand) outputNodeDriverInfo(node *api.Node) {
+	c.Ui.Output(c.Colorize().Color("\n[bold]Drivers"))
+
+	size := len(node.Drivers)
+	nodeDrivers := make([]string, 0, size+1)
+
+	nodeDrivers = append(nodeDrivers, "Driver|Detected|Healthy|Message|Time")
+
+	drivers := make([]string, 0, len(node.Drivers))
+	for driver := range node.Drivers {
+		drivers = append(drivers, driver)
+	}
+	sort.Strings(drivers)
+
+	for _, driver := range drivers {
+		info := node.Drivers[driver]
+		timestamp := formatTime(info.UpdateTime)
+		nodeDrivers = append(nodeDrivers, fmt.Sprintf("%s|%v|%v|%s|%s", driver, info.Detected, info.Healthy, info.HealthDescription, timestamp))
+	}
+	c.Ui.Output(formatList(nodeDrivers))
+}
+
+func (c *NodeStatusCommand) outputNodeStatusEvents(node *api.Node) {
+	c.Ui.Output(c.Colorize().Color("\n[bold]Node Events"))
+	c.outputNodeEvent(node.Events)
+}
+
+func (c *NodeStatusCommand) outputNodeEvent(events []*api.NodeEvent) {
+	size := len(events)
+	nodeEvents := make([]string, size+1)
+	if c.verbose {
+		nodeEvents[0] = "Time|Subsystem|Message|Details"
+	} else {
+		nodeEvents[0] = "Time|Subsystem|Message"
+	}
+
+	for i, event := range events {
+		timestamp := formatTime(event.Timestamp)
+		subsystem := formatEventSubsystem(event.Subsystem, event.Details["driver"])
+		msg := event.Message
+		if c.verbose {
+			details := formatEventDetails(event.Details)
+			nodeEvents[size-i] = fmt.Sprintf("%s|%s|%s|%s", timestamp, subsystem, msg, details)
+		} else {
+			nodeEvents[size-i] = fmt.Sprintf("%s|%s|%s", timestamp, subsystem, msg)
+		}
+	}
+	c.Ui.Output(formatList(nodeEvents))
+}
+
+func formatEventSubsystem(subsystem, driverName string) string {
+	if driverName == "" {
+		return subsystem
+	}
+
+	// If this event is for a driver, append the driver name to make the message
+	// clearer
+	return fmt.Sprintf("Driver: %s", driverName)
+}
+
+func formatEventDetails(details map[string]string) string {
+	output := make([]string, 0, len(details))
+	for k, v := range details {
+		output = append(output, fmt.Sprintf("%s: %s ", k, v))
+	}
+	return strings.Join(output, ", ")
+}
+
 func (c *NodeStatusCommand) formatAttributes(node *api.Node) {
 	// Print the attributes
 	keys := make([]string, len(node.Attributes))
@@ -316,6 +521,24 @@ func (c *NodeStatusCommand) formatAttributes(node *api.Node) {
 	c.Ui.Output(formatKV(attributes))
 }
 
+func (c *NodeStatusCommand) formatMeta(node *api.Node) {
+	// Print the meta
+	keys := make([]string, 0, len(node.Meta))
+	for k := range node.Meta {
+		keys = append(keys, k)
+	}
+	sort.Strings(keys)
+
+	var meta []string
+	for _, k := range keys {
+		if k != "" {
+			meta = append(meta, fmt.Sprintf("%s|%s", k, node.Meta[k]))
+		}
+	}
+	c.Ui.Output(c.Colorize().Color("\n[bold]Meta[reset]"))
+	c.Ui.Output(formatKV(meta))
+}
+
 func (c *NodeStatusCommand) printCpuStats(hostStats *api.HostStats) {
 	l := len(hostStats.CPU)
 	for i, cpuStat := range hostStats.CPU {
@@ -374,26 +597,6 @@ func getRunningAllocs(client *api.Client, nodeID string) ([]*api.Allocation, err
 	return allocs, err
 }
 
-// getAllocs returns information about every running allocation on the node
-func getAllocs(client *api.Client, node *api.Node, length int) ([]string, error) {
-	var allocs []string
-	// Query the node allocations
-	nodeAllocs, _, err := client.Nodes().Allocations(node.ID, nil)
-	// Format the allocations
-	allocs = make([]string, len(nodeAllocs)+1)
-	allocs[0] = "ID|Eval ID|Job ID|Task Group|Desired Status|Client Status"
-	for i, alloc := range nodeAllocs {
-		allocs[i+1] = fmt.Sprintf("%s|%s|%s|%s|%s|%s",
-			limit(alloc.ID, length),
-			limit(alloc.EvalID, length),
-			alloc.JobID,
-			alloc.TaskGroup,
-			alloc.DesiredStatus,
-			alloc.ClientStatus)
-	}
-	return allocs, err
-}
-
 // getAllocatedResources returns the resource usage of the node.
 func getAllocatedResources(client *api.Client, runningAllocs []*api.Allocation, node *api.Node) []string {
 	// Compute the total
@@ -402,23 +605,23 @@ func getAllocatedResources(client *api.Client, runningAllocs []*api.Allocation,
 	// Get Resources
 	var cpu, mem, disk, iops int
 	for _, alloc := range runningAllocs {
-		cpu += alloc.Resources.CPU
-		mem += alloc.Resources.MemoryMB
-		disk += alloc.Resources.DiskMB
-		iops += alloc.Resources.IOPS
+		cpu += *alloc.Resources.CPU
+		mem += *alloc.Resources.MemoryMB
+		disk += *alloc.Resources.DiskMB
+		iops += *alloc.Resources.IOPS
 	}
 
 	resources := make([]string, 2)
 	resources[0] = "CPU|Memory|Disk|IOPS"
-	resources[1] = fmt.Sprintf("%v/%v|%v/%v|%v/%v|%v/%v",
+	resources[1] = fmt.Sprintf("%d/%d MHz|%s/%s|%s/%s|%d/%d",
 		cpu,
-		total.CPU,
+		*total.CPU,
 		humanize.IBytes(uint64(mem*bytesPerMegabyte)),
-		humanize.IBytes(uint64(total.MemoryMB*bytesPerMegabyte)),
+		humanize.IBytes(uint64(*total.MemoryMB*bytesPerMegabyte)),
 		humanize.IBytes(uint64(disk*bytesPerMegabyte)),
-		humanize.IBytes(uint64(total.DiskMB*bytesPerMegabyte)),
+		humanize.IBytes(uint64(*total.DiskMB*bytesPerMegabyte)),
 		iops,
-		total.IOPS)
+		*total.IOPS)
 
 	return resources
 }
@@ -433,10 +636,10 @@ func computeNodeTotalResources(node *api.Node) api.Resources {
 	if res == nil {
 		res = &api.Resources{}
 	}
-	total.CPU = r.CPU - res.CPU
-	total.MemoryMB = r.MemoryMB - res.MemoryMB
-	total.DiskMB = r.DiskMB - res.DiskMB
-	total.IOPS = r.IOPS - res.IOPS
+	total.CPU = helper.IntToPtr(*r.CPU - *res.CPU)
+	total.MemoryMB = helper.IntToPtr(*r.MemoryMB - *res.MemoryMB)
+	total.DiskMB = helper.IntToPtr(*r.DiskMB - *res.DiskMB)
+	total.IOPS = helper.IntToPtr(*r.IOPS - *res.IOPS)
 	return total
 }
 
@@ -461,11 +664,11 @@ func getActualResources(client *api.Client, runningAllocs []*api.Allocation, nod
 
 	resources := make([]string, 2)
 	resources[0] = "CPU|Memory"
-	resources[1] = fmt.Sprintf("%v/%v|%v/%v",
+	resources[1] = fmt.Sprintf("%v/%d MHz|%v/%v",
 		math.Floor(cpu),
-		total.CPU,
+		*total.CPU,
 		humanize.IBytes(mem),
-		humanize.IBytes(uint64(total.MemoryMB*bytesPerMegabyte)))
+		humanize.IBytes(uint64(*total.MemoryMB*bytesPerMegabyte)))
 
 	return resources, nil
 }
@@ -480,22 +683,66 @@ func getHostResources(hostStats *api.HostStats, node *api.Node) ([]string, error
 	// calculate disk usage
 	storageDevice := node.Attributes["unique.storage.volume"]
 	var diskUsed, diskSize uint64
+	var physical bool
 	for _, disk := range hostStats.DiskStats {
 		if disk.Device == storageDevice {
 			diskUsed = disk.Used
 			diskSize = disk.Size
+			physical = true
 		}
 	}
 
 	resources = make([]string, 2)
 	resources[0] = "CPU|Memory|Disk"
-	resources[1] = fmt.Sprintf("%v/%v|%v/%v|%v/%v",
-		math.Floor(hostStats.CPUTicksConsumed),
-		node.Resources.CPU,
-		humanize.IBytes(hostStats.Memory.Used),
-		humanize.IBytes(hostStats.Memory.Total),
-		humanize.IBytes(diskUsed),
-		humanize.IBytes(diskSize),
-	)
+	if physical {
+		resources[1] = fmt.Sprintf("%v/%d MHz|%s/%s|%s/%s",
+			math.Floor(hostStats.CPUTicksConsumed),
+			*node.Resources.CPU,
+			humanize.IBytes(hostStats.Memory.Used),
+			humanize.IBytes(hostStats.Memory.Total),
+			humanize.IBytes(diskUsed),
+			humanize.IBytes(diskSize),
+		)
+	} else {
+		// If non-physical device are used, output device name only,
+		// since nomad doesn't collect the stats data.
+		resources[1] = fmt.Sprintf("%v/%d MHz|%s/%s|(%s)",
+			math.Floor(hostStats.CPUTicksConsumed),
+			*node.Resources.CPU,
+			humanize.IBytes(hostStats.Memory.Used),
+			humanize.IBytes(hostStats.Memory.Total),
+			storageDevice,
+		)
+	}
 	return resources, nil
 }
+
+// formatNodeStubList is used to return a table format of a list of node stubs.
+func formatNodeStubList(nodes []*api.NodeListStub, verbose bool) string {
+	// Return error if no nodes are found
+	if len(nodes) == 0 {
+		return ""
+	}
+	// Truncate the id unless full length is requested
+	length := shortId
+	if verbose {
+		length = fullId
+	}
+
+	// Format the nodes list that matches the prefix so that the user
+	// can create a more specific request
+	out := make([]string, len(nodes)+1)
+	out[0] = "ID|DC|Name|Class|Drain|Eligibility|Status"
+	for i, node := range nodes {
+		out[i+1] = fmt.Sprintf("%s|%s|%s|%s|%v|%s|%s",
+			limit(node.ID, length),
+			node.Datacenter,
+			node.Name,
+			node.NodeClass,
+			node.Drain,
+			node.SchedulingEligibility,
+			node.Status)
+	}
+
+	return formatList(out)
+}
