@@ -1,11 +1,14 @@
 package nomad
 
 import (
+	"fmt"
 	"testing"
 	"time"
 
 	"github.com/hashicorp/nomad/nomad/mock"
 	"github.com/hashicorp/nomad/nomad/structs"
+	"github.com/hashicorp/nomad/testutil"
+	"github.com/stretchr/testify/require"
 )
 
 var (
@@ -15,18 +18,39 @@ var (
 	}
 )
 
+func testBrokerConfig() *Config {
+	config := DefaultConfig()
+
+	// Tune the Nack timeout
+	config.EvalNackTimeout = 5 * time.Second
+
+	// Tune the Nack delay
+	config.EvalNackInitialReenqueueDelay = 5 * time.Millisecond
+	config.EvalNackSubsequentReenqueueDelay = 50 * time.Millisecond
+	return config
+}
+
 func testBroker(t *testing.T, timeout time.Duration) *EvalBroker {
-	if timeout == 0 {
-		timeout = 5 * time.Second
+	config := testBrokerConfig()
+
+	if timeout != 0 {
+		config.EvalNackTimeout = timeout
 	}
-	b, err := NewEvalBroker(timeout, 3)
+
+	return testBrokerFromConfig(t, config)
+}
+
+func testBrokerFromConfig(t *testing.T, c *Config) *EvalBroker {
+	b, err := NewEvalBroker(c.EvalNackTimeout, c.EvalNackInitialReenqueueDelay, c.EvalNackSubsequentReenqueueDelay, 3)
 	if err != nil {
 		t.Fatalf("err: %v", err)
 	}
+
 	return b
 }
 
 func TestEvalBroker_Enqueue_Dequeue_Nack_Ack(t *testing.T) {
+	t.Parallel()
 	b := testBroker(t, 0)
 
 	// Enqueue, but broker is disabled!
@@ -126,19 +150,28 @@ func TestEvalBroker_Enqueue_Dequeue_Nack_Ack(t *testing.T) {
 	}
 
 	// Check the stats
-	stats = b.Stats()
-	if stats.TotalReady != 1 {
-		t.Fatalf("bad: %#v", stats)
-	}
-	if stats.TotalUnacked != 0 {
-		t.Fatalf("bad: %#v", stats)
-	}
-	if stats.ByScheduler[eval.Type].Ready != 1 {
-		t.Fatalf("bad: %#v", stats)
-	}
-	if stats.ByScheduler[eval.Type].Unacked != 0 {
-		t.Fatalf("bad: %#v", stats)
-	}
+	testutil.WaitForResult(func() (bool, error) {
+		stats = b.Stats()
+		if stats.TotalReady != 1 {
+			return false, fmt.Errorf("bad: %#v", stats)
+		}
+		if stats.TotalUnacked != 0 {
+			return false, fmt.Errorf("bad: %#v", stats)
+		}
+		if stats.TotalWaiting != 0 {
+			return false, fmt.Errorf("bad: %#v", stats)
+		}
+		if stats.ByScheduler[eval.Type].Ready != 1 {
+			return false, fmt.Errorf("bad: %#v", stats)
+		}
+		if stats.ByScheduler[eval.Type].Unacked != 0 {
+			return false, fmt.Errorf("bad: %#v", stats)
+		}
+
+		return true, nil
+	}, func(e error) {
+		t.Fatal(e)
+	})
 
 	// Dequeue should work again
 	out2, token2, err := b.Dequeue(defaultSched, time.Second)
@@ -192,28 +225,204 @@ func TestEvalBroker_Enqueue_Dequeue_Nack_Ack(t *testing.T) {
 	}
 }
 
+func TestEvalBroker_Nack_Delay(t *testing.T) {
+	t.Parallel()
+	b := testBroker(t, 0)
+
+	// Enqueue, but broker is disabled!
+	b.SetEnabled(true)
+	eval := mock.Eval()
+	b.Enqueue(eval)
+
+	// Dequeue should work
+	out, token, err := b.Dequeue(defaultSched, time.Second)
+	if err != nil {
+		t.Fatalf("err: %v", err)
+	}
+	if out != eval {
+		t.Fatalf("bad : %#v", out)
+	}
+
+	// Nack back into the queue
+	err = b.Nack(eval.ID, token)
+	if err != nil {
+		t.Fatalf("err: %v", err)
+	}
+
+	if _, ok := b.Outstanding(out.ID); ok {
+		t.Fatalf("should not be outstanding")
+	}
+
+	// Check the stats to ensure that it is waiting
+	stats := b.Stats()
+	if stats.TotalReady != 0 {
+		t.Fatalf("bad: %#v", stats)
+	}
+	if stats.TotalUnacked != 0 {
+		t.Fatalf("bad: %#v", stats)
+	}
+	if stats.TotalWaiting != 1 {
+		t.Fatalf("bad: %#v", stats)
+	}
+	if stats.ByScheduler[eval.Type].Ready != 0 {
+		t.Fatalf("bad: %#v", stats)
+	}
+	if stats.ByScheduler[eval.Type].Unacked != 0 {
+		t.Fatalf("bad: %#v", stats)
+	}
+
+	// Now wait for it to be re-enqueued
+	testutil.WaitForResult(func() (bool, error) {
+		stats = b.Stats()
+		if stats.TotalReady != 1 {
+			return false, fmt.Errorf("bad: %#v", stats)
+		}
+		if stats.TotalUnacked != 0 {
+			return false, fmt.Errorf("bad: %#v", stats)
+		}
+		if stats.TotalWaiting != 0 {
+			return false, fmt.Errorf("bad: %#v", stats)
+		}
+		if stats.ByScheduler[eval.Type].Ready != 1 {
+			return false, fmt.Errorf("bad: %#v", stats)
+		}
+		if stats.ByScheduler[eval.Type].Unacked != 0 {
+			return false, fmt.Errorf("bad: %#v", stats)
+		}
+
+		return true, nil
+	}, func(e error) {
+		t.Fatal(e)
+	})
+
+	// Dequeue should work again
+	out2, token2, err := b.Dequeue(defaultSched, time.Second)
+	if err != nil {
+		t.Fatalf("err: %v", err)
+	}
+	if out2 != eval {
+		t.Fatalf("bad : %#v", out2)
+	}
+	if token2 == token {
+		t.Fatalf("should get a new token")
+	}
+
+	// Capture the time
+	start := time.Now()
+
+	// Nack back into the queue
+	err = b.Nack(eval.ID, token2)
+	if err != nil {
+		t.Fatalf("err: %v", err)
+	}
+
+	// Now wait for it to be re-enqueued
+	testutil.WaitForResult(func() (bool, error) {
+		stats = b.Stats()
+		if stats.TotalReady != 1 {
+			return false, fmt.Errorf("bad: %#v", stats)
+		}
+		if stats.TotalUnacked != 0 {
+			return false, fmt.Errorf("bad: %#v", stats)
+		}
+		if stats.TotalWaiting != 0 {
+			return false, fmt.Errorf("bad: %#v", stats)
+		}
+		if stats.ByScheduler[eval.Type].Ready != 1 {
+			return false, fmt.Errorf("bad: %#v", stats)
+		}
+		if stats.ByScheduler[eval.Type].Unacked != 0 {
+			return false, fmt.Errorf("bad: %#v", stats)
+		}
+
+		return true, nil
+	}, func(e error) {
+		t.Fatal(e)
+	})
+
+	delay := time.Now().Sub(start)
+	if delay < b.subsequentNackDelay {
+		t.Fatalf("bad: delay was %v; want at least %v", delay, b.subsequentNackDelay)
+	}
+
+	// Dequeue should work again
+	out3, token3, err := b.Dequeue(defaultSched, time.Second)
+	if err != nil {
+		t.Fatalf("err: %v", err)
+	}
+	if out3 != eval {
+		t.Fatalf("bad : %#v", out3)
+	}
+	if token3 == token || token3 == token2 {
+		t.Fatalf("should get a new token")
+	}
+
+	// Ack finally
+	err = b.Ack(eval.ID, token3)
+	if err != nil {
+		t.Fatalf("err: %v", err)
+	}
+
+	if _, ok := b.Outstanding(out.ID); ok {
+		t.Fatalf("should not be outstanding")
+	}
+
+	// Check the stats
+	stats = b.Stats()
+	if stats.TotalReady != 0 {
+		t.Fatalf("bad: %#v", stats)
+	}
+	if stats.TotalUnacked != 0 {
+		t.Fatalf("bad: %#v", stats)
+	}
+	if stats.ByScheduler[eval.Type].Ready != 0 {
+		t.Fatalf("bad: %#v", stats)
+	}
+	if stats.ByScheduler[eval.Type].Unacked != 0 {
+		t.Fatalf("bad: %#v", stats)
+	}
+}
+
 func TestEvalBroker_Serialize_DuplicateJobID(t *testing.T) {
+	t.Parallel()
 	b := testBroker(t, 0)
 	b.SetEnabled(true)
 
+	ns1 := "namespace-one"
+	ns2 := "namespace-two"
 	eval := mock.Eval()
+	eval.Namespace = ns1
 	b.Enqueue(eval)
 
 	eval2 := mock.Eval()
 	eval2.JobID = eval.JobID
+	eval2.Namespace = ns1
 	eval2.CreateIndex = eval.CreateIndex + 1
 	b.Enqueue(eval2)
 
 	eval3 := mock.Eval()
 	eval3.JobID = eval.JobID
+	eval3.Namespace = ns1
 	eval3.CreateIndex = eval.CreateIndex + 2
 	b.Enqueue(eval3)
 
+	eval4 := mock.Eval()
+	eval4.JobID = eval.JobID
+	eval4.Namespace = ns2
+	eval4.CreateIndex = eval.CreateIndex + 3
+	b.Enqueue(eval4)
+
+	eval5 := mock.Eval()
+	eval5.JobID = eval.JobID
+	eval5.Namespace = ns2
+	eval5.CreateIndex = eval.CreateIndex + 4
+	b.Enqueue(eval5)
+
 	stats := b.Stats()
-	if stats.TotalReady != 1 {
+	if stats.TotalReady != 2 {
 		t.Fatalf("bad: %#v", stats)
 	}
-	if stats.TotalBlocked != 2 {
+	if stats.TotalBlocked != 3 {
 		t.Fatalf("bad: %#v", stats)
 	}
 
@@ -228,13 +437,13 @@ func TestEvalBroker_Serialize_DuplicateJobID(t *testing.T) {
 
 	// Check the stats
 	stats = b.Stats()
-	if stats.TotalReady != 0 {
+	if stats.TotalReady != 1 {
 		t.Fatalf("bad: %#v", stats)
 	}
 	if stats.TotalUnacked != 1 {
 		t.Fatalf("bad: %#v", stats)
 	}
-	if stats.TotalBlocked != 2 {
+	if stats.TotalBlocked != 3 {
 		t.Fatalf("bad: %#v", stats)
 	}
 
@@ -246,13 +455,13 @@ func TestEvalBroker_Serialize_DuplicateJobID(t *testing.T) {
 
 	// Check the stats
 	stats = b.Stats()
-	if stats.TotalReady != 1 {
+	if stats.TotalReady != 2 {
 		t.Fatalf("bad: %#v", stats)
 	}
 	if stats.TotalUnacked != 0 {
 		t.Fatalf("bad: %#v", stats)
 	}
-	if stats.TotalBlocked != 1 {
+	if stats.TotalBlocked != 2 {
 		t.Fatalf("bad: %#v", stats)
 	}
 
@@ -265,6 +474,84 @@ func TestEvalBroker_Serialize_DuplicateJobID(t *testing.T) {
 		t.Fatalf("bad : %#v", out)
 	}
 
+	// Check the stats
+	stats = b.Stats()
+	if stats.TotalReady != 1 {
+		t.Fatalf("bad: %#v", stats)
+	}
+	if stats.TotalUnacked != 1 {
+		t.Fatalf("bad: %#v", stats)
+	}
+	if stats.TotalBlocked != 2 {
+		t.Fatalf("bad: %#v", stats)
+	}
+
+	// Ack out
+	err = b.Ack(eval2.ID, token)
+	if err != nil {
+		t.Fatalf("err: %v", err)
+	}
+
+	// Check the stats
+	stats = b.Stats()
+	if stats.TotalReady != 2 {
+		t.Fatalf("bad: %#v", stats)
+	}
+	if stats.TotalUnacked != 0 {
+		t.Fatalf("bad: %#v", stats)
+	}
+	if stats.TotalBlocked != 1 {
+		t.Fatalf("bad: %#v", stats)
+	}
+
+	// Dequeue should work
+	out, token, err = b.Dequeue(defaultSched, time.Second)
+	if err != nil {
+		t.Fatalf("err: %v", err)
+	}
+	if out != eval3 {
+		t.Fatalf("bad : %#v", out)
+	}
+
+	// Check the stats
+	stats = b.Stats()
+	if stats.TotalReady != 1 {
+		t.Fatalf("bad: %#v", stats)
+	}
+	if stats.TotalUnacked != 1 {
+		t.Fatalf("bad: %#v", stats)
+	}
+	if stats.TotalBlocked != 1 {
+		t.Fatalf("bad: %#v", stats)
+	}
+
+	// Ack out
+	err = b.Ack(eval3.ID, token)
+	if err != nil {
+		t.Fatalf("err: %v", err)
+	}
+
+	// Check the stats
+	stats = b.Stats()
+	if stats.TotalReady != 1 {
+		t.Fatalf("bad: %#v", stats)
+	}
+	if stats.TotalUnacked != 0 {
+		t.Fatalf("bad: %#v", stats)
+	}
+	if stats.TotalBlocked != 1 {
+		t.Fatalf("bad: %#v", stats)
+	}
+
+	// Dequeue should work
+	out, token, err = b.Dequeue(defaultSched, time.Second)
+	if err != nil {
+		t.Fatalf("err: %v", err)
+	}
+	if out != eval4 {
+		t.Fatalf("bad : %#v", out)
+	}
+
 	// Check the stats
 	stats = b.Stats()
 	if stats.TotalReady != 0 {
@@ -278,7 +565,7 @@ func TestEvalBroker_Serialize_DuplicateJobID(t *testing.T) {
 	}
 
 	// Ack out
-	err = b.Ack(eval2.ID, token)
+	err = b.Ack(eval4.ID, token)
 	if err != nil {
 		t.Fatalf("err: %v", err)
 	}
@@ -300,7 +587,7 @@ func TestEvalBroker_Serialize_DuplicateJobID(t *testing.T) {
 	if err != nil {
 		t.Fatalf("err: %v", err)
 	}
-	if out != eval3 {
+	if out != eval5 {
 		t.Fatalf("bad : %#v", out)
 	}
 
@@ -317,7 +604,7 @@ func TestEvalBroker_Serialize_DuplicateJobID(t *testing.T) {
 	}
 
 	// Ack out
-	err = b.Ack(eval3.ID, token)
+	err = b.Ack(eval5.ID, token)
 	if err != nil {
 		t.Fatalf("err: %v", err)
 	}
@@ -336,6 +623,7 @@ func TestEvalBroker_Serialize_DuplicateJobID(t *testing.T) {
 }
 
 func TestEvalBroker_Enqueue_Disable(t *testing.T) {
+	t.Parallel()
 	b := testBroker(t, 0)
 
 	// Enqueue
@@ -360,6 +648,7 @@ func TestEvalBroker_Enqueue_Disable(t *testing.T) {
 }
 
 func TestEvalBroker_Dequeue_Timeout(t *testing.T) {
+	t.Parallel()
 	b := testBroker(t, 0)
 	b.SetEnabled(true)
 
@@ -380,6 +669,7 @@ func TestEvalBroker_Dequeue_Timeout(t *testing.T) {
 }
 
 func TestEvalBroker_Dequeue_Empty_Timeout(t *testing.T) {
+	t.Parallel()
 	b := testBroker(t, 0)
 	b.SetEnabled(true)
 	doneCh := make(chan struct{}, 1)
@@ -416,6 +706,7 @@ func TestEvalBroker_Dequeue_Empty_Timeout(t *testing.T) {
 
 // Ensure higher priority dequeued first
 func TestEvalBroker_Dequeue_Priority(t *testing.T) {
+	t.Parallel()
 	b := testBroker(t, 0)
 	b.SetEnabled(true)
 
@@ -449,6 +740,7 @@ func TestEvalBroker_Dequeue_Priority(t *testing.T) {
 
 // Ensure FIFO at fixed priority
 func TestEvalBroker_Dequeue_FIFO(t *testing.T) {
+	t.Parallel()
 	b := testBroker(t, 0)
 	b.SetEnabled(true)
 	NUM := 100
@@ -470,9 +762,10 @@ func TestEvalBroker_Dequeue_FIFO(t *testing.T) {
 
 // Ensure fairness between schedulers
 func TestEvalBroker_Dequeue_Fairness(t *testing.T) {
+	t.Parallel()
 	b := testBroker(t, 0)
 	b.SetEnabled(true)
-	NUM := 100
+	NUM := 1000
 
 	for i := 0; i < NUM; i++ {
 		eval1 := mock.Eval()
@@ -503,7 +796,7 @@ func TestEvalBroker_Dequeue_Fairness(t *testing.T) {
 
 		// This will fail randomly at times. It is very hard to
 		// test deterministically that its acting randomly.
-		if counter >= 25 || counter <= -25 {
+		if counter >= 250 || counter <= -250 {
 			t.Fatalf("unlikely sequence: %d", counter)
 		}
 	}
@@ -511,6 +804,7 @@ func TestEvalBroker_Dequeue_Fairness(t *testing.T) {
 
 // Ensure we get unblocked
 func TestEvalBroker_Dequeue_Blocked(t *testing.T) {
+	t.Parallel()
 	b := testBroker(t, 0)
 	b.SetEnabled(true)
 
@@ -549,6 +843,7 @@ func TestEvalBroker_Dequeue_Blocked(t *testing.T) {
 
 // Ensure we nack in a timely manner
 func TestEvalBroker_Nack_Timeout(t *testing.T) {
+	t.Parallel()
 	b := testBroker(t, 5*time.Millisecond)
 	b.SetEnabled(true)
 
@@ -584,7 +879,8 @@ func TestEvalBroker_Nack_Timeout(t *testing.T) {
 
 // Ensure we nack in a timely manner
 func TestEvalBroker_Nack_TimeoutReset(t *testing.T) {
-	b := testBroker(t, 5*time.Millisecond)
+	t.Parallel()
+	b := testBroker(t, 50*time.Millisecond)
 	b.SetEnabled(true)
 
 	// Enqueue
@@ -601,8 +897,8 @@ func TestEvalBroker_Nack_TimeoutReset(t *testing.T) {
 		t.Fatalf("bad: %v", out)
 	}
 
-	// Reset in 2 milliseconds
-	time.Sleep(2 * time.Millisecond)
+	// Reset in 20 milliseconds
+	time.Sleep(20 * time.Millisecond)
 	if err := b.OutstandingReset(out.ID, token); err != nil {
 		t.Fatalf("err: %v", err)
 	}
@@ -618,13 +914,14 @@ func TestEvalBroker_Nack_TimeoutReset(t *testing.T) {
 	}
 
 	// Check the nack timer
-	if diff := end.Sub(start); diff < 7*time.Millisecond {
+	if diff := end.Sub(start); diff < 75*time.Millisecond {
 		t.Fatalf("bad: %#v", diff)
 	}
 }
 
 func TestEvalBroker_PauseResumeNackTimeout(t *testing.T) {
-	b := testBroker(t, 5*time.Millisecond)
+	t.Parallel()
+	b := testBroker(t, 50*time.Millisecond)
 	b.SetEnabled(true)
 
 	// Enqueue
@@ -641,14 +938,14 @@ func TestEvalBroker_PauseResumeNackTimeout(t *testing.T) {
 		t.Fatalf("bad: %v", out)
 	}
 
-	// Pause in 2 milliseconds
-	time.Sleep(2 * time.Millisecond)
+	// Pause in 20 milliseconds
+	time.Sleep(20 * time.Millisecond)
 	if err := b.PauseNackTimeout(out.ID, token); err != nil {
 		t.Fatalf("err: %v", err)
 	}
 
 	go func() {
-		time.Sleep(2 * time.Millisecond)
+		time.Sleep(20 * time.Millisecond)
 		if err := b.ResumeNackTimeout(out.ID, token); err != nil {
 			t.Fatalf("err: %v", err)
 		}
@@ -665,12 +962,13 @@ func TestEvalBroker_PauseResumeNackTimeout(t *testing.T) {
 	}
 
 	// Check the nack timer
-	if diff := end.Sub(start); diff < 9*time.Millisecond {
+	if diff := end.Sub(start); diff < 95*time.Millisecond {
 		t.Fatalf("bad: %#v", diff)
 	}
 }
 
 func TestEvalBroker_DeliveryLimit(t *testing.T) {
+	t.Parallel()
 	b := testBroker(t, 0)
 	b.SetEnabled(true)
 
@@ -760,6 +1058,7 @@ func TestEvalBroker_DeliveryLimit(t *testing.T) {
 }
 
 func TestEvalBroker_AckAtDeliveryLimit(t *testing.T) {
+	t.Parallel()
 	b := testBroker(t, 0)
 	b.SetEnabled(true)
 
@@ -802,6 +1101,7 @@ func TestEvalBroker_AckAtDeliveryLimit(t *testing.T) {
 
 // Ensure fairness between schedulers
 func TestEvalBroker_Wait(t *testing.T) {
+	t.Parallel()
 	b := testBroker(t, 0)
 	b.SetEnabled(true)
 
@@ -820,7 +1120,7 @@ func TestEvalBroker_Wait(t *testing.T) {
 	}
 
 	// Let the wait elapse
-	time.Sleep(15 * time.Millisecond)
+	time.Sleep(20 * time.Millisecond)
 
 	// Verify ready
 	stats = b.Stats()
@@ -841,8 +1141,55 @@ func TestEvalBroker_Wait(t *testing.T) {
 	}
 }
 
+// Ensure that delayed evaluations work as expected
+func TestEvalBroker_WaitUntil(t *testing.T) {
+	t.Parallel()
+	require := require.New(t)
+	b := testBroker(t, 0)
+	b.SetEnabled(true)
+
+	now := time.Now()
+	// Create a few of evals with WaitUntil set
+	eval1 := mock.Eval()
+	eval1.WaitUntil = now.Add(1 * time.Second)
+	eval1.CreateIndex = 1
+	b.Enqueue(eval1)
+
+	eval2 := mock.Eval()
+	eval2.WaitUntil = now.Add(100 * time.Millisecond)
+	// set CreateIndex to use as a tie breaker when eval2
+	// and eval3 are both in the pending evals heap
+	eval2.CreateIndex = 2
+	b.Enqueue(eval2)
+
+	eval3 := mock.Eval()
+	eval3.WaitUntil = now.Add(20 * time.Millisecond)
+	eval3.CreateIndex = 1
+	b.Enqueue(eval3)
+	require.Equal(3, b.stats.TotalWaiting)
+	// sleep enough for two evals to be ready
+	time.Sleep(200 * time.Millisecond)
+
+	// first dequeue should return eval3
+	out, _, err := b.Dequeue(defaultSched, time.Second)
+	require.Nil(err)
+	require.Equal(eval3, out)
+
+	// second dequeue should return eval2
+	out, _, err = b.Dequeue(defaultSched, time.Second)
+	require.Nil(err)
+	require.Equal(eval2, out)
+
+	// third dequeue should return eval1
+	out, _, err = b.Dequeue(defaultSched, 2*time.Second)
+	require.Nil(err)
+	require.Equal(eval1, out)
+	require.Equal(0, b.stats.TotalWaiting)
+}
+
 // Ensure that priority is taken into account when enqueueing many evaluations.
 func TestEvalBroker_EnqueueAll_Dequeue_Fair(t *testing.T) {
+	t.Parallel()
 	b := testBroker(t, 0)
 	b.SetEnabled(true)
 
@@ -887,6 +1234,7 @@ func TestEvalBroker_EnqueueAll_Dequeue_Fair(t *testing.T) {
 }
 
 func TestEvalBroker_EnqueueAll_Requeue_Ack(t *testing.T) {
+	t.Parallel()
 	b := testBroker(t, 0)
 	b.SetEnabled(true)
 
@@ -943,6 +1291,7 @@ func TestEvalBroker_EnqueueAll_Requeue_Ack(t *testing.T) {
 }
 
 func TestEvalBroker_EnqueueAll_Requeue_Nack(t *testing.T) {
+	t.Parallel()
 	b := testBroker(t, 0)
 	b.SetEnabled(true)
 
@@ -976,14 +1325,62 @@ func TestEvalBroker_EnqueueAll_Requeue_Nack(t *testing.T) {
 	}
 
 	// Check stats again as this should cause the re-enqueued one to be dropped
-	stats = b.Stats()
-	if stats.TotalReady != 1 {
-		t.Fatalf("bad: %#v", stats)
-	}
-	if stats.TotalUnacked != 0 {
-		t.Fatalf("bad: %#v", stats)
-	}
-	if len(b.requeue) != 0 {
-		t.Fatalf("bad: %#v", b.requeue)
-	}
+	testutil.WaitForResult(func() (bool, error) {
+		stats = b.Stats()
+		if stats.TotalReady != 1 {
+			return false, fmt.Errorf("bad: %#v", stats)
+		}
+		if stats.TotalUnacked != 0 {
+			return false, fmt.Errorf("bad: %#v", stats)
+		}
+		if len(b.requeue) != 0 {
+			return false, fmt.Errorf("bad: %#v", b.requeue)
+		}
+
+		return true, nil
+	}, func(e error) {
+		t.Fatal(e)
+	})
+}
+
+func TestEvalBroker_NamespacedJobs(t *testing.T) {
+	t.Parallel()
+	b := testBroker(t, 0)
+	b.SetEnabled(true)
+
+	// Create evals with the same jobid and different namespace
+	jobId := "test-jobID"
+
+	eval1 := mock.Eval()
+	eval1.JobID = jobId
+	eval1.Namespace = "n1"
+	b.Enqueue(eval1)
+
+	// This eval should not block
+	eval2 := mock.Eval()
+	eval2.JobID = jobId
+	eval2.Namespace = "default"
+	b.Enqueue(eval2)
+
+	// This eval should block
+	eval3 := mock.Eval()
+	eval3.JobID = jobId
+	eval3.Namespace = "default"
+	b.Enqueue(eval3)
+
+	require := require.New(t)
+	out1, _, err := b.Dequeue(defaultSched, 5*time.Millisecond)
+	require.Nil(err)
+	require.Equal(eval1.ID, out1.ID)
+
+	out2, _, err := b.Dequeue(defaultSched, 5*time.Millisecond)
+	require.Nil(err)
+	require.Equal(eval2.ID, out2.ID)
+
+	out3, _, err := b.Dequeue(defaultSched, 5*time.Millisecond)
+	require.Nil(err)
+	require.Nil(out3)
+
+	require.Equal(1, len(b.blocked))
+
 }
