@@ -23,6 +23,13 @@ type FeasibleIterator interface {
 	Reset()
 }
 
+// JobContextualIterator is an iterator that can have the job and task group set
+// on it.
+type ContextualIterator interface {
+	SetJob(*structs.Job)
+	SetTaskGroup(*structs.TaskGroup)
+}
+
 // FeasibilityChecker is used to check if a single node meets feasibility
 // constraints.
 type FeasibilityChecker interface {
@@ -122,6 +129,22 @@ func (c *DriverChecker) Feasible(option *structs.Node) bool {
 func (c *DriverChecker) hasDrivers(option *structs.Node) bool {
 	for driver := range c.drivers {
 		driverStr := fmt.Sprintf("driver.%s", driver)
+
+		// COMPAT: Remove in 0.10: As of Nomad 0.8, nodes have a DriverInfo that
+		// corresponds with every driver. As a Nomad server might be on a later
+		// version than a Nomad client, we need to check for compatibility here
+		// to verify the client supports this.
+		if driverInfo, ok := option.Drivers[driver]; ok {
+			if driverInfo == nil {
+				c.ctx.Logger().
+					Printf("[WARN] scheduler.DriverChecker: node %v has no driver info set for %v",
+						option.ID, driver)
+				return false
+			}
+
+			return driverInfo.Detected && driverInfo.Healthy
+		}
+
 		value, ok := option.Attributes[driverStr]
 		if !ok {
 			return false
@@ -142,12 +165,10 @@ func (c *DriverChecker) hasDrivers(option *structs.Node) bool {
 	return true
 }
 
-// ProposedAllocConstraintIterator is a FeasibleIterator which returns nodes that
-// match constraints that are not static such as Node attributes but are
-// effected by proposed alloc placements. Examples are distinct_hosts and
-// tenancy constraints. This is used to filter on job and task group
-// constraints.
-type ProposedAllocConstraintIterator struct {
+// DistinctHostsIterator is a FeasibleIterator which returns nodes that pass the
+// distinct_hosts constraint. The constraint ensures that multiple allocations
+// do not exist on the same node.
+type DistinctHostsIterator struct {
 	ctx    Context
 	source FeasibleIterator
 	tg     *structs.TaskGroup
@@ -159,44 +180,47 @@ type ProposedAllocConstraintIterator struct {
 	jobDistinctHosts bool
 }
 
-// NewProposedAllocConstraintIterator creates a ProposedAllocConstraintIterator
-// from a source.
-func NewProposedAllocConstraintIterator(ctx Context, source FeasibleIterator) *ProposedAllocConstraintIterator {
-	return &ProposedAllocConstraintIterator{
+// NewDistinctHostsIterator creates a DistinctHostsIterator from a source.
+func NewDistinctHostsIterator(ctx Context, source FeasibleIterator) *DistinctHostsIterator {
+	return &DistinctHostsIterator{
 		ctx:    ctx,
 		source: source,
 	}
 }
 
-func (iter *ProposedAllocConstraintIterator) SetTaskGroup(tg *structs.TaskGroup) {
+func (iter *DistinctHostsIterator) SetTaskGroup(tg *structs.TaskGroup) {
 	iter.tg = tg
 	iter.tgDistinctHosts = iter.hasDistinctHostsConstraint(tg.Constraints)
 }
 
-func (iter *ProposedAllocConstraintIterator) SetJob(job *structs.Job) {
+func (iter *DistinctHostsIterator) SetJob(job *structs.Job) {
 	iter.job = job
 	iter.jobDistinctHosts = iter.hasDistinctHostsConstraint(job.Constraints)
 }
 
-func (iter *ProposedAllocConstraintIterator) hasDistinctHostsConstraint(constraints []*structs.Constraint) bool {
+func (iter *DistinctHostsIterator) hasDistinctHostsConstraint(constraints []*structs.Constraint) bool {
 	for _, con := range constraints {
 		if con.Operand == structs.ConstraintDistinctHosts {
 			return true
 		}
 	}
+
 	return false
 }
 
-func (iter *ProposedAllocConstraintIterator) Next() *structs.Node {
+func (iter *DistinctHostsIterator) Next() *structs.Node {
 	for {
 		// Get the next option from the source
 		option := iter.source.Next()
 
-		// Hot-path if the option is nil or there are no distinct_hosts constraints.
-		if option == nil || !(iter.jobDistinctHosts || iter.tgDistinctHosts) {
+		// Hot-path if the option is nil or there are no distinct_hosts or
+		// distinct_property constraints.
+		hosts := iter.jobDistinctHosts || iter.tgDistinctHosts
+		if option == nil || !hosts {
 			return option
 		}
 
+		// Check if the host constraints are satisfied
 		if !iter.satisfiesDistinctHosts(option) {
 			iter.ctx.Metrics().FilterNode(option, structs.ConstraintDistinctHosts)
 			continue
@@ -208,7 +232,7 @@ func (iter *ProposedAllocConstraintIterator) Next() *structs.Node {
 
 // satisfiesDistinctHosts checks if the node satisfies a distinct_hosts
 // constraint either specified at the job level or the TaskGroup level.
-func (iter *ProposedAllocConstraintIterator) satisfiesDistinctHosts(option *structs.Node) bool {
+func (iter *DistinctHostsIterator) satisfiesDistinctHosts(option *structs.Node) bool {
 	// Check if there is no constraint set.
 	if !(iter.jobDistinctHosts || iter.tgDistinctHosts) {
 		return true
@@ -237,10 +261,115 @@ func (iter *ProposedAllocConstraintIterator) satisfiesDistinctHosts(option *stru
 	return true
 }
 
-func (iter *ProposedAllocConstraintIterator) Reset() {
+func (iter *DistinctHostsIterator) Reset() {
 	iter.source.Reset()
 }
 
+// DistinctPropertyIterator is a FeasibleIterator which returns nodes that pass the
+// distinct_property constraint. The constraint ensures that multiple allocations
+// do not use the same value of the given property.
+type DistinctPropertyIterator struct {
+	ctx    Context
+	source FeasibleIterator
+	tg     *structs.TaskGroup
+	job    *structs.Job
+
+	hasDistinctPropertyConstraints bool
+	jobPropertySets                []*propertySet
+	groupPropertySets              map[string][]*propertySet
+}
+
+// NewDistinctPropertyIterator creates a DistinctPropertyIterator from a source.
+func NewDistinctPropertyIterator(ctx Context, source FeasibleIterator) *DistinctPropertyIterator {
+	return &DistinctPropertyIterator{
+		ctx:               ctx,
+		source:            source,
+		groupPropertySets: make(map[string][]*propertySet),
+	}
+}
+
+func (iter *DistinctPropertyIterator) SetTaskGroup(tg *structs.TaskGroup) {
+	iter.tg = tg
+
+	// Build the property set at the taskgroup level
+	if _, ok := iter.groupPropertySets[tg.Name]; !ok {
+		for _, c := range tg.Constraints {
+			if c.Operand != structs.ConstraintDistinctProperty {
+				continue
+			}
+
+			pset := NewPropertySet(iter.ctx, iter.job)
+			pset.SetTGConstraint(c, tg.Name)
+			iter.groupPropertySets[tg.Name] = append(iter.groupPropertySets[tg.Name], pset)
+		}
+	}
+
+	// Check if there is a distinct property
+	iter.hasDistinctPropertyConstraints = len(iter.jobPropertySets) != 0 || len(iter.groupPropertySets[tg.Name]) != 0
+}
+
+func (iter *DistinctPropertyIterator) SetJob(job *structs.Job) {
+	iter.job = job
+
+	// Build the property set at the job level
+	for _, c := range job.Constraints {
+		if c.Operand != structs.ConstraintDistinctProperty {
+			continue
+		}
+
+		pset := NewPropertySet(iter.ctx, job)
+		pset.SetJobConstraint(c)
+		iter.jobPropertySets = append(iter.jobPropertySets, pset)
+	}
+}
+
+func (iter *DistinctPropertyIterator) Next() *structs.Node {
+	for {
+		// Get the next option from the source
+		option := iter.source.Next()
+
+		// Hot path if there is nothing to check
+		if option == nil || !iter.hasDistinctPropertyConstraints {
+			return option
+		}
+
+		// Check if the constraints are met
+		if !iter.satisfiesProperties(option, iter.jobPropertySets) ||
+			!iter.satisfiesProperties(option, iter.groupPropertySets[iter.tg.Name]) {
+			continue
+		}
+
+		return option
+	}
+}
+
+// satisfiesProperties returns whether the option satisfies the set of
+// properties. If not it will be filtered.
+func (iter *DistinctPropertyIterator) satisfiesProperties(option *structs.Node, set []*propertySet) bool {
+	for _, ps := range set {
+		if satisfies, reason := ps.SatisfiesDistinctProperties(option, iter.tg.Name); !satisfies {
+			iter.ctx.Metrics().FilterNode(option, reason)
+			return false
+		}
+	}
+
+	return true
+}
+
+func (iter *DistinctPropertyIterator) Reset() {
+	iter.source.Reset()
+
+	for _, ps := range iter.jobPropertySets {
+		ps.PopulateProposed()
+	}
+
+	for _, sets := range iter.groupPropertySets {
+		for _, ps := range sets {
+			ps.PopulateProposed()
+		}
+	}
+}
+
 // ConstraintChecker is a FeasibilityChecker which returns nodes that match a
 // given set of constraints. This is used to filter on job, task group, and task
 // constraints.
@@ -327,7 +456,7 @@ func resolveConstraintTarget(target string, node *structs.Node) (interface{}, bo
 func checkConstraint(ctx Context, operand string, lVal, rVal interface{}) bool {
 	// Check for constraints not handled by this checker.
 	switch operand {
-	case structs.ConstraintDistinctHosts:
+	case structs.ConstraintDistinctHosts, structs.ConstraintDistinctProperty:
 		return true
 	default:
 		break
@@ -344,6 +473,8 @@ func checkConstraint(ctx Context, operand string, lVal, rVal interface{}) bool {
 		return checkVersionConstraint(ctx, lVal, rVal)
 	case structs.ConstraintRegex:
 		return checkRegexpConstraint(ctx, lVal, rVal)
+	case structs.ConstraintSetContains:
+		return checkSetContainsConstraint(ctx, lVal, rVal)
 	default:
 		return false
 	}
@@ -451,6 +582,38 @@ func checkRegexpConstraint(ctx Context, lVal, rVal interface{}) bool {
 	return re.MatchString(lStr)
 }
 
+// checkSetContainsConstraint is used to see if the left hand side contains the
+// string on the right hand side
+func checkSetContainsConstraint(ctx Context, lVal, rVal interface{}) bool {
+	// Ensure left-hand is string
+	lStr, ok := lVal.(string)
+	if !ok {
+		return false
+	}
+
+	// Regexp must be a string
+	rStr, ok := rVal.(string)
+	if !ok {
+		return false
+	}
+
+	input := strings.Split(lStr, ",")
+	lookup := make(map[string]struct{}, len(input))
+	for _, in := range input {
+		cleaned := strings.TrimSpace(in)
+		lookup[cleaned] = struct{}{}
+	}
+
+	for _, r := range strings.Split(rStr, ",") {
+		cleaned := strings.TrimSpace(r)
+		if _, ok := lookup[cleaned]; !ok {
+			return false
+		}
+	}
+
+	return true
+}
+
 // FeasibilityWrapper is a FeasibleIterator which wraps both job and task group
 // FeasibilityCheckers in which feasibility checking can be skipped if the
 // computed node class has previously been marked as eligible or ineligible.
