@@ -1,27 +1,23 @@
 #!/usr/bin/env bash
 
 function install_go() {
-	local go_version=1.10.2
+	local go_version=1.10.7
 	local download=
 
 	download="https://storage.googleapis.com/golang/go${go_version}.linux-amd64.tar.gz"
 
-	if [ -d /usr/local/go ] ; then
-		return
-	fi
-
 	wget -q -O /tmp/go.tar.gz ${download}
 
 	tar -C /tmp -xf /tmp/go.tar.gz
 	sudo mv /tmp/go /usr/local
-	sudo chown -R root:root /usr/local/go
+	#sudo chown -R root:root /usr/local/go
 }
 
 install_go
 	
 # Ensure that the GOPATH tree is owned by vagrant:vagrant
-mkdir -p /opt/gopath
-chown -R vagrant:vagrant /opt/gopath
+#mkdir -p /opt/gopath
+#chown -R vagrant:vagrant /opt/gopath
 
 # Ensure Go is on PATH
 if [ ! -e /usr/bin/go ] ; then
